Anvil = LibStub("AceAddon-3.0"):NewAddon("Anvil", "AceConsole-3.0", "AceEvent-3.0")
AnvilFrames = LibStub("AceGUI-3.0")
local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)
local _G = _G
local anvilVersion = GetAddOnMetadata("Anvil", "Version")

function Anvil:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("AnvilDB", {
		global = {
			archaeology = true,
			archaeologyScale = 1,
			archaeologyIsMovable = false,
			collections = true,
			collectionsWidth = 1,
			collectionsIsMovable = false,
			collectionsScale = 1,
			spellbook = true,
			spellbookScale = 1,
			spellbookWidth = 2,
			spellbookHeight = 6,
			spellbookIsMovable = false,
			vendorScale = 1,
			vendorWidth = 2,
			vendorHeight = 5,
			vendorHideExtra = true,
			vendorIsMovable = false,
			minimap = {hide = false},
			startup = true
		}
	})
	self:RegisterChatCommand("anvil", "ChatCommand")
	AnvilLauncher = LDB:NewDataObject("Anvil", {
		type = "launcher",
		icon = "136241",
		OnClick = function(_, button)
			if button == "RightButton" then
				InterfaceOptionsFrame_OpenToCategory(Anvil.optionsFrame)
			end
		end,
		OnTooltipShow = function(tt)
			tt:AddLine("|cFFFF0000Anvil "..anvilVersion)
			tt:AddLine("|cFFFF8080Right Click|cFFFFFFFF to display the options panel")
		end})
	LDBIcon:Register("Anvil", AnvilLauncher, self.db.global.minimap)
end

function Anvil:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_ENABLED", Anvil_Notify)
	self:RegisterEvent("PLAYER_REGEN_DISABLED", Anvil_Notify)
	if not IsAddOnLoaded("Blizzard_Collections") then
		LoadAddOn("Blizzard_Collections")
	end
	CollectionsJournal:SetScale(self.db.global.collectionsScale)
	local AnvilOptions = {
		name = "Anvil",
		handler = Anvil,
		type = 'group',
		childGroups = "tab",
		args = {
			Archaeology = {
				name = 'Archaeology',
				handler = Anvil,
				order = 1,
				type = 'group',
				args = {
					frames = {
						order = 1,
						type = 'header',
						name = 'Choose whether or not Anvil enhances the Archaeology Frame'
					},
					archaeology = {
						order = 2,
						type = 'toggle',
						name = 'Enable',
						desc = 'Allow Anvil to enhance the Archaeology UI frame',
						get = function() return self.db.global.archaeology end,
						set = 'Set'
					},
					archaeologyDesc = {
						order = 3,
						type = 'description',
						name = 'Enables the Anvil version of the Archaeology frame.\n'
					},
					archaeologyScale = {
						order = 4,
						type = 'range',
						name = 'UI Scale',
						desc = 'Set the scale of the Archaeology frame',
						get = function() return self.db.global.archaeologyScale end,
						set = 'VariableSet',
						min = .75,
						max = 1.25,
						step = .05,
						disabled = function() return not self.db.global.archaeology end
					},
					archaeologyIsMovable = {
						order = 5,
						type = 'toggle',
						name = 'Movable',
						desc = 'Make the Archaeology frame movable',
						get = function() return self.db.global.archaeologyIsMovable end,
						set = 'VariableSet',
						disabled = function() return not self.db.global.archaeology end
					}
				}
			},
			Collections = {
				name = 'Collections',
				handler = Anvil,
				order = 2,
				type = 'group',
				args = {
					frames = {
						order = 1,
						type = 'header',
						name = 'Choose How Anvil Enhances the  Collections Journal'
					},
					collections = {
						order = 2,
						type = 'toggle',
						name = 'Enable',
						desc = 'Allow Anvil to enhance the Collections Journal',
						get = function() return self.db.global.collections end,
						set = 'Set',
						disabled = function() return InCombatLockdown() end
					},
					collectionsDesc = {
						order = 3,
						type = 'description',
						name = 'Allows customization of the Collections Journal\n|cFFFF0000WARNING|cFFFFFFFF: The Anvil Collections Journal will only work out of combat.  In combat, it is replaced by the standard Collections Journal.\n'
					},
					collectionsScale = {
						order = 4,
						type = 'range',
						name = 'UI Scale',
						desc = 'Set the scale of the Collections Journal',
						get = function() return self.db.global.collectionsScale end,
						set = 'VariableSet',
						min = .75,
						max = 1.25,
						step = .05,
						disabled = function() if InCombatLockdown() or not self.db.global.collections then return true else return false end end
					},
					collectionsWidth = {
						order = 5,
						type = 'range',
						name = 'Collections Width',
						desc = 'Adjust the width factor for the Collections Journal',
						get = function() return self.db.global.collectionsWidth end,
						set = 'VariableSet',
						min = 1,
						max = 5,
						step = 1,
						disabled = function() if InCombatLockdown() or not self.db.global.collections then return true else return false end end
					},
					collectionsIsMovable = {
						order = 6,
						type = 'toggle',
						name = 'Movable',
						desc = 'Make the Collections Journal movable',
						get = function() return self.db.global.collectionsIsMovable end,
						set = 'VariableSet',
						disabled = function() if InCombatLockdown() or not self.db.global.collections then return true else return false end end
					}
				}
			},
			Spellbook = {
				name = 'Spellbook',
				handler = Anvil,
				order = 2,
				type = 'group',
				args = {
					frames = {
						order = 1,
						type = 'header',
						name = 'Choose How Anvil Enhances the Spellbook Frame'
					},
					spellbook = {
						order = 2,
						type = 'toggle',
						name = 'Enable',
						desc = 'Allow Anvil to enhance the Spellbook UI frame',
						get = function() return self.db.global.spellbook end,
						set = 'Set',
						disabled = function() return InCombatLockdown() end
					},
					spellbookDesc = {
						order = 3,
						type = 'description',
						name = 'Allows customization of the Spellbook frame\n|cFFFF0000WARNING|cFFFFFFFF: The Anvil Spellbook will only work out of combat.  In combat, it is replaced by the standard Spellbook.\n'
					},
					spellbookScale = {
						order = 4,
						type = 'range',
						name = 'UI Scale',
						desc = 'Set the scale of the Spellbook window',
						get = function() return self.db.global.spellbookScale end,
						set = 'VariableSet',
						min = .75,
						max = 1.25,
						step = .05,
						disabled = function() if InCombatLockdown() or not self.db.global.spellbook then return true else return false end end
					},
					spellbookWidth = {
						order = 5,
						type = 'range',
						name = 'Spellbook Columns',
						desc = 'Adjust how many columns the Spellbook frame uses',
						get = function() return self.db.global.spellbookWidth end,
						set = 'VariableSet',
						min = 2,
						max = 10,
						step = 1,
						disabled = function() if InCombatLockdown() or not self.db.global.spellbook then return true else return false end end
					},
					spellbookHeight = {
						order = 6,
						type = 'range',
						name = 'Spellbook Rows',
						desc = 'Adjusts how many rows the Spellbook frame uses',
						get = function() return self.db.global.spellbookHeight end,
						set = 'VariableSet',
						min = 2,
						max = 10,
						step = 1,
						disabled = function() if InCombatLockdown() or not self.db.global.spellbook then return true else return false end end
					},
					spellbookIsMovable = {
						order = 7,
						type = 'toggle',
						name = 'Movable',
						desc = 'Make the Spellbook Frame movable',
						get = function() return self.db.global.spellbookIsMovable end,
						set = 'VariableSet',
						disabled = function() if InCombatLockdown() or not self.db.global.spellbook then return true else return false end end
					}
				}
			},
			Vendor = {
				name = 'Vendor',
				handler = Anvil,
				order = 3,
				type = 'group',
				args = {
					frames = {
						order = 1,
						type = 'header',
						name = 'Choose How Anvil Enhances the Vendor Frame'
					},
					vendorScale = {
						order = 2,
						type = 'range',
						name = 'UI Scale',
						desc = 'Set the scale of the Vendor window',
						get = function() return self.db.global.vendorScale end,
						set = 'VariableSet',
						min = .75,
						max = 1.25,
						step = .05
					},
					vendorWidth = {
						order = 3,
						type = 'range',
						name = 'Vendor Columns',
						desc = 'Adjust how many columns the Vendor frame uses',
						get = function() return self.db.global.vendorWidth end,
						set = 'VariableSet',
						min = 2,
						max = 10,
						step = 1
					},
					vendorHeight = {
						order = 4,
						type = 'range',
						name = 'Vendor Rows',
						desc = 'Adjusts how many rows the Vendor frame uses',
						get = function() return self.db.global.vendorHeight end,
						set = 'VariableSet',
						min = 2,
						max = 10,
						step = 1
					},
					vendorHideExtra = {
						order = 5,
						type = 'toggle',
						name = 'Hide extra buttons',
						desc = 'Hide extra Vendor item buttons',
						get = function() return self.db.global.vendorHideExtra end,
						set = 'VariableSet'
					},
					vendorIsMovable = {
						order = 6,
						type = 'toggle',
						name = 'Movable',
						desc = 'Make the Vendor Frame movable',
						get = function() return self.db.global.vendorIsMovable end,
						set = 'VariableSet'
					}
				}
			},
			Miscellaneous = {
				name = "Miscellaneous",
				handler = Anvil,
				order = 4,
				type = "group",
				args = {
					minimap = {
						order = 1,
						type = 'toggle',
						name = "Minimap Button",
						desc = 'Show the minimap button',
						get = function() return not self.db.global.minimap.hide end,
						set = 'VariableSet'
					},
					startup = {
						order = 2,
						type = 'toggle',
						name = 'Show Version Loaded',
						desc = 'Show the one-line startup text that displays Anvil version loaded.',
						get = function() return self.db.global.startup end,
						set = 'VariableSet'
					},
					aboutHeader = {
						order = 3,
						type = 'header',
						name = 'About'
					},
					version = {
						order = 4,
						type = 'description',
						name = "Version: |cFFFF0000"..anvilVersion.."|r"
					},
					author = {
						order = 5,
						type = 'description',
						name = 'by |cFFFF0000InSo(v)nIaX|r'
					}
				}
			}
		}
	}
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Anvil", AnvilOptions)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Anvil")
	if self.db.global.startup then
		print("|cFFFF0000Anvil|r: v"..anvilVersion.." Loaded.")
	end
end

function Anvil_Notify()
	LibStub("AceConfigRegistry-3.0"):NotifyChange("Anvil")
end

function Anvil:MakeMovable(var, func)
	if Anvil.db.global[var] then
		_G[func]:SetMovable(true)
		_G[func]:RegisterForDrag("LeftButton")
		_G[func]:SetClampedToScreen(true)
		_G[func]:SetScript('OnDragStart', function() _G[func]:StartMoving() end)
		_G[func]:SetScript('OnDragStop', function() _G[func]:StopMovingOrSizing() end)
	else
		_G[func]:SetMovable(false)
		_G[func]:RegisterForDrag()
	end
end

function Anvil:ChatCommand(input)
	if not input or input:trim() == "" or input == "options" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(Anvil, "Anvil", input)
	end
end

function Anvil:Set(arg, value)
	self.db.global[arg[2]] = value
	local module = "Anvil"..arg[2]:gsub("^%l", string.upper)
	if value == true then
		_G[module]:Enable()
	else
		_G[module]:Disable()
	end
end

function Anvil:VariableSet(arg, value)
	if arg[2] == "minimap" then
		self.db.global.minimap.hide = not value
		if self.db.global.minimap.hide then
			LDBIcon:Hide("Anvil")
		else
			LDBIcon:Show("Anvil")
		end
		return
	else
		self.db.global[arg[2]] = value
		if (arg[2] == "vendorScale" or arg[2] == "vendorWidth" or arg[2] == "vendorHeight" or arg[2] == "vendorHideExtra" or arg[2] == "vendorIsMovable") then
			AnvilVendor:ResizeVendorFrame()
		elseif arg[2] == "archaeologyIsMovable" then
			self:MakeMovable("archaeologyIsMovable", "ArchaeologyFrame")
		elseif arg[2] == "archaeologyScale" then
			ArchaeologyFrame:SetScale(self.db.global.archaeologyScale)
		elseif (arg[2] == "spellbookScale" or arg[2] == "spellbookWidth" or arg[2] == "spellbookHeight" or arg[2] == "spellbookIsMovable") and self.db.global.spellbook then
			AnvilSpellbook:ResizeSpellbookFrame()
		elseif arg[2] == "collectionsScale" and self.db.global.collections then
			AnvilCollectionsJournal:SetScale(self.db.global.collectionsScale)
			AnvilWardrobeCollectionFrame.ItemsCollectionFrame:RefreshCameras()
			AnvilWardrobeCollectionDressUpFrame.DressUpModel:RefreshCamera()
		elseif arg[2] == "collectionsIsMovable" and self.db.global.collections then
			self:MakeMovable("collectionsIsMovable", "AnvilCollectionsJournal")
		elseif arg[2] == "collectionsWidth" and self.db.global.collections then
			AnvilCollectionsJournal:SetSize(447+Anvil.db.global.collectionsWidth*256, 632)
			AnvilMounts:ResizeMountJournal()
			AnvilPets:ResizePetJournal()
			AnvilToys:ResizeToyBox()
			AnvilHeirloomsJournal:RebuildLayoutData()
			AnvilHeirloomsJournal:LayoutCurrentPage()
			AnvilHeirloomsJournal:RefreshView()
			if not AnvilWardrobeCollectionFrame.ItemsCollectionFrame.activeCategory then
				AnvilWardrobeCollectionFrame.ItemsCollectionFrame:SetActiveSlot("HEADSLOT", LE_TRANSMOG_TYPE_APPEARANCE);
			end
			AnvilWardrobeCollectionFrame.ItemsCollectionFrame:RefreshModelsSlot()
			AnvilWardrobeCollectionFrame.ItemsCollectionFrame:RefreshVisualsList()
			AnvilWardrobeCollectionFrame.ItemsCollectionFrame:UpdateItems()
			AnvilWardrobeCollectionFrameScrollFrame:ResizeSetsScrollFrame()
			AnvilCollections_UpdateSelectedTab(AnvilCollectionsJournal)
		end
	end
end
