AnvilCollections = Anvil:NewModule("AnvilCollections", "AceConsole-3.0", "AceEvent-3.0")
local _G = _G
AnvilCollectionsJournal = CreateFrame("Frame", "AnvilCollectionsJournal", UIParent, "AnvilCollectionsJournalTemplate")
AnvilCollectionsJournal:SetFrameStrata("HIGH")
local postCombatLaunch

local function ShouldShowAnvilHeirloomTabHelpTip()
	if GetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_HEIRLOOM_JOURNAL_TAB) or IsKioskModeEnabled() then
		return false;
	end
	if AnvilPetJournal_HelpPlate and HelpPlate_IsShowing(AnvilPetJournal_HelpPlate) then
		return false;
	end
	return C_Heirloom.ShouldShowHeirloomHelp();
end

local function ShouldShowAnvilWardrobeTabHelpTip()
	if GetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TRANSMOG_JOURNAL_TAB) or IsKioskModeEnabled() then
		return false;
	end
	if AnvilPetJournal_HelpPlate and HelpPlate_IsShowing(AnvilPetJournal_HelpPlate) then
		return false;
	end
	if AnvilCollectionsJournal.AnvilHeirloomTabHelpBox:IsShown() then
		return false;
	end
	return true;
end

function AnvilCollections:OnEnable()
	if not Anvil.db.global.collections then
		return
	else
		AnvilMounts:Enable()
		AnvilPets:Enable()
		AnvilToys:Enable()
		AnvilCollectionsJournal:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 16, -95)
		AnvilCollectionsJournal:RegisterEvent("PLAYER_REGEN_ENABLED")
		AnvilCollectionsJournal:RegisterEvent("PLAYER_REGEN_DISABLED")
		hooksecurefunc("ToggleCollectionsJournal", AnvilCollections_Toggle)
		AnvilCollectionsJournal:SetSize(447+Anvil.db.global.collectionsWidth*256, 632)
		tinsert(UISpecialFrames, "AnvilCollectionsJournal")
		AnvilCollectionsJournal:EnableMouse(true)
	end
end

function AnvilCollections:OnDisable()
	AnvilCollectionsJournal:UnregisterAllEvents()
	AnvilMounts:Disable()
	AnvilPets:Disable()
	AnvilToys:Disable()
end

function AnvilCollections_OnEvent(self, event, ...)
	if ( event == "PLAYER_REGEN_DISABLED" ) then
		if AnvilCollectionsJournal:IsVisible() then
			ToggleCollectionsJournal(PanelTemplates_GetSelectedTab(AnvilCollectionsJournal))
			postCombatLaunch = true
		end
	elseif ( event == "PLAYER_REGEN_ENABLED" ) then
		if postCombatLaunch then
			ToggleCollectionsJournal(PanelTemplates_GetSelectedTab(AnvilCollectionsJournal))
			postCombatLaunch = false
		end
	end
end

function AnvilCollections_Toggle(whichFrame)
	if (not Anvil.db.global.collections or InCombatLockdown()) then
		return
	else
		CollectionsJournalCloseButton:Click()
		collectgarbage("collect")
		if ( whichFrame ) then
			if ( AnvilCollectionsJournal:IsShown() and whichFrame == PanelTemplates_GetSelectedTab(AnvilCollectionsJournal) ) then
				HideUIPanel(AnvilCollectionsJournal);
			else
				ShowUIPanel(AnvilCollectionsJournal);
				AnvilCollections_SetTab(AnvilCollectionsJournal, whichFrame);
			end
		else
			ToggleFrame(AnvilCollectionsJournal);
		end
	end
end

function AnvilCollectionsJournal_OnShow(self)
	HideUIPanel(WardrobeFrame);
	CollectionsMicroButtonAlert:Hide();
	MicroButtonPulseStop(CollectionsMicroButton);

	AnvilCollectionsJournal_UpdateSelectedTab(self);
	UpdateMicroButtons();
end

function AnvilCollectionsJournal_OnHide(self)
	collectgarbage("collect")
	UpdateMicroButtons();
	CollectionsMicroButton:EvaluateAlertVisibility();
end

function AnvilCollections_SetTab(self, tab)
	PanelTemplates_SetTab(self, tab);
	AnvilCollections_UpdateSelectedTab(self);
	collectgarbage("collect")
end

function AnvilCollections_UpdateSelectedTab(self)
	local selected = PanelTemplates_GetSelectedTab(self);
	if (not AnvilCollections_ValidateTab(selected)) then
		PanelTemplates_SetTab(self, 1);
		selected = 1;
	end
	if ( selected == 1 ) then
		AnvilCollectionsJournalTitleText:SetText(MOUNTS);
	elseif (selected == 2 ) then
		AnvilCollectionsJournalTitleText:SetText(PET_JOURNAL);
	elseif (selected == 3 ) then
		AnvilCollectionsJournalTitleText:SetText(TOY_BOX);
	elseif (selected == 4 ) then
		AnvilHeirloomsJournal:RebuildLayoutData()
		AnvilCollectionsJournalTitleText:SetText(HEIRLOOMS);
	elseif (selected == 5 ) then
		AnvilCollectionsJournalTitleText:SetText(WARDROBE);
	end
	if not InCombatLockdown() then
		AnvilMountJournal:SetShown(selected == 1);
		AnvilPetJournal:SetShown(selected == 2);
		AnvilToyBox:SetShown(selected == 3);
		AnvilHeirloomsJournal:SetShown(selected == 4);
		AnvilWardrobeCollectionFrame:SetShown(selected == 5)
	end

	self.AnvilHeirloomTabHelpBox:SetShown(ShouldShowAnvilHeirloomTabHelpTip());
	self.AnvilWardrobeTabHelpBox:SetShown(ShouldShowAnvilWardrobeTabHelpTip());
	collectgarbage("collect")
end

function AnvilCollections_ValidateTab(tabNum)
	return true;
end

function AnvilCollectionsSpellButton_OnLoad(self, updateFunction)
	self:RegisterForDrag("LeftButton");
	self:RegisterForClicks("LeftButtonUp", "RightButtonUp");

	self.updateFunction = updateFunction;
end

function AnvilCollectionsButton_OnEvent(self, event, ...)
	if GameTooltip:GetOwner() == self then
		self:GetScript("OnEnter")(self);
	end

	self.updateFunction(self);
end

function AnvilCollectionsSpellButton_OnShow(self)
	self:RegisterEvent("SPELLS_CHANGED");
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM");

	self.updateFunction(self);
end

function AnvilCollectionsSpellButton_OnHide(self)
	self:UnregisterEvent("SPELLS_CHANGED");
	self:UnregisterEvent("SPELL_UPDATE_COOLDOWN");
	self:UnregisterEvent("UPDATE_SHAPESHIFT_FORM");
end

function AnvilCollectionsSpellButton_UpdateCooldown(self)
	if (self.itemID == -1 or self.itemID == nil) then
		return;
	end

	local cooldown = self.cooldown;
	local start, duration, enable = GetItemCooldown(self.itemID);
	if (cooldown and start and duration) then
		if (enable) then
			cooldown:Hide();
		else
			cooldown:Show();
		end
		CooldownFrame_Set(cooldown, start, duration, enable);
	else
		cooldown:Hide();
	end
end

AnvilCollectionsWrappedModelSceneMixin = {};

function AnvilCollectionsWrappedModelSceneMixin:IsUnwrapAnimating()
	return self.isUnwrapping;
end

function AnvilCollectionsWrappedModelSceneMixin:NeedsFanfare()
	return self.needsFanFare;
end

function AnvilCollectionsWrappedModelSceneMixin:StartUnwrapAnimation(OnFinishedCallback)
	if not self:NeedsFanfare() or self:IsUnwrapAnimating() then
		return;
	end

	local wrappedActor = self:GetActorByTag("wrapped");
	local unwrappedActor = self:GetActorByTag("unwrapped");

	if unwrappedActor and wrappedActor then
		self.isUnwrapping = true;
		self.needsFanFare = false;

		self.UnwrapAnim.WrappedAnim:SetTarget(wrappedActor);
		self.UnwrapAnim.UnwrappedAnim:SetTarget(unwrappedActor);

		self.UnwrapAnim:Play();

		wrappedActor:SetAnimation(148);

		PlaySound(SOUNDKIT.UI_STORE_UNWRAP);

		C_Timer.After(.8, function()
			for actor in self:EnumerateActiveActors() do
				actor:ApplySpellVisualKit(73393, true);
			end
		end)

		C_Timer.After(1.6, function()
			self.isUnwrapping = nil;
			if OnFinishedCallback then
				OnFinishedCallback();
			end
		end)
	end
end

function AnvilCollectionsWrappedModelSceneMixin:PrepareForFanfare(needsFanFare)
	self.needsFanFare = needsFanFare;

	local wrappedActor = self:GetActorByTag("wrapped");
	local unwrappedActor = self:GetActorByTag("unwrapped");
	if wrappedActor and unwrappedActor then
		wrappedActor:SetModelByCreatureDisplayID(COLLECTIONS_FANFARE_CREATURE_DISPLAY_ID);
		if self:NeedsFanfare() then
			wrappedActor:Show();
			if not self:IsUnwrapAnimating() then
				unwrappedActor:SetAlpha(0);
				wrappedActor:SetAnimation(0);
				wrappedActor:SetAlpha(1);
			end
		else
			wrappedActor:Hide();
			if not self:IsUnwrapAnimating() then
				unwrappedActor:SetAlpha(1);
			end
		end
	end
end

function AnvilCollectionsWrappedModelSceneMixin:OnShow()
	self:SetLightAmbientColor(self.normalIntensity, self.normalIntensity, self.normalIntensity);
end

function AnvilCollectionsWrappedModelSceneMixin:OnMouseEnter()
	if self:NeedsFanfare() then
		self:SetLightAmbientColor(self.highlightIntensity, self.highlightIntensity, self.highlightIntensity);
	else
		self:SetLightAmbientColor(self.normalIntensity, self.normalIntensity, self.normalIntensity);
	end
end

function AnvilCollectionsWrappedModelSceneMixin:OnMouseLeave()
	self:SetLightAmbientColor(self.normalIntensity, self.normalIntensity, self.normalIntensity);
end

AnvilCollectionsPagingMixin = { };

function AnvilCollectionsPagingMixin:OnLoad()
	self.currentPage = 1;
	self.maxPages = 1;
	self:Update();
end

function AnvilCollectionsPagingMixin:SetMaxPages(maxPages)
	maxPages = math.max(maxPages, 1);
	if ( self.maxPages == maxPages ) then
		return;
	end
	self.maxPages= maxPages;
	if ( self.maxPages < self.currentPage ) then
		self.currentPage = self.maxPages;
	end
	self:Update();
end

function AnvilCollectionsPagingMixin:GetMaxPages()
	return self.maxPages;
end

function AnvilCollectionsPagingMixin:SetCurrentPage(page, userAction)
	page = Clamp(page, 1, self.maxPages);
	if ( self.currentPage ~= page ) then
		self.currentPage = page;
		self:Update();
		if ( self:GetParent().OnPageChanged ) then
			self:GetParent():OnPageChanged(userAction);
		end
	end
end

function AnvilCollectionsPagingMixin:GetCurrentPage()
	return self.currentPage;
end

function AnvilCollectionsPagingMixin:NextPage()
	self:SetCurrentPage(self.currentPage + 1, true);
	if PanelTemplates_GetSelectedTab(AnvilCollectionsJournal) == 3 then
		_G[self:GetParent():GetName().."_UpdateButtons"]()
	end
end

function AnvilCollectionsPagingMixin:PreviousPage()
	self:SetCurrentPage(self.currentPage - 1, true);
	if PanelTemplates_GetSelectedTab(AnvilCollectionsJournal) == 3 then
		_G[self:GetParent():GetName().."_UpdateButtons"]()
	end
end

function AnvilCollectionsPagingMixin:OnMouseWheel(delta)
	if ( delta > 0 ) then
		self:PreviousPage();
	else
		self:NextPage();
	end
end

function AnvilCollectionsPagingMixin:Update()
	self.PageText:SetFormattedText(COLLECTION_PAGE_NUMBER, self.currentPage, self.maxPages);
	if ( self.currentPage <= 1 ) then
		self.PrevPageButton:Disable();
	else
		self.PrevPageButton:Enable();
	end
	if ( self.currentPage >= self.maxPages ) then
		self.NextPageButton:Disable();
	else
		self.NextPageButton:Enable();
	end
end
