AnvilMounts = Anvil:NewModule("AnvilMounts", "AceConsole-3.0", "AceEvent-3.0")
local _G = _G

local MOUNT_BUTTON_HEIGHT = 46;
local PLAYER_MOUNT_LEVEL = 20;
local SUMMON_RANDOM_FAVORITE_MOUNT_SPELL = 150544;

local MOUNT_FACTION_TEXTURES = {
	[0] = "MountJournalIcons-Horde",
	[1] = "MountJournalIcons-Alliance"
};

function AnvilMounts:OnEnable()
	if not Anvil.db.global.collections then
		return
	else
		AnvilMountJournal:RegisterEvent("COMPANION_LEARNED");
		AnvilMountJournal:RegisterEvent("COMPANION_UNLEARNED");
		AnvilMountJournal:RegisterEvent("COMPANION_UPDATE");
		AnvilMountJournal:RegisterEvent("MOUNT_JOURNAL_USABILITY_CHANGED");
		AnvilMountJournal:RegisterEvent("MOUNT_JOURNAL_SEARCH_UPDATED");
		AnvilMountJournal:RegisterEvent("UI_MODEL_SCENE_INFO_UPDATED");
		AnvilMountJournal.ListScrollFrame.update = AnvilMountJournal_UpdateMountList;
		AnvilMountJournal.ListScrollFrame.scrollBar.doNotHide = true;
		HybridScrollFrame_CreateButtons(AnvilMountJournal.ListScrollFrame, "AnvilMountListButtonTemplate", 44, 0);
		self:ResizeMountJournal()
		UIDropDownMenu_Initialize(AnvilMountJournal.mountOptionsMenu, AnvilMountOptionsMenu_Init, "MENU");
	end
end

function AnvilMounts:OnDisable()
	self:UnregisterAllEvents()
end

function AnvilMounts:ResizeMountJournal()
	AnvilMountJournal.ListScrollFrame.stepSize = Anvil.db.global.collectionsWidth*46
	if AnvilMountJournal.ListScrollFrame.buttons then
		wipe(AnvilMountJournal.ListScrollFrame.buttons)
	else
		AnvilMountJournal.ListScrollFrame.buttons = {}
	end
	for i=1, 55 do
		if i<=11*Anvil.db.global.collectionsWidth then
			if not _G["AnvilMountJournalListScrollFrameButton"..i] then
				_G["AnvilMountJournalListScrollFrameButton"..i] = CreateFrame("Button", "AnvilMountJournalListScrollFrameButton" .. i, AnvilMountJournalListScrollFrameScrollChild, "AnvilMountListButtonTemplate")
			end
			tinsert(AnvilMountJournal.ListScrollFrame.buttons, _G["AnvilMountJournalListScrollFrameButton"..i])
		elseif _G["AnvilMountJournalListScrollFrameButton"..i] then
			_G["AnvilMountJournalListScrollFrameButton"..i]:Hide()
		end
	end
	AnvilMountJournal_UpdateMountList()
	for i=1, 11 do
		for j=1, Anvil.db.global.collectionsWidth do
			_G["AnvilMountJournalListScrollFrameButton"..(i-1)*Anvil.db.global.collectionsWidth+j]:SetPoint("TOPLEFT", AnvilMountJournal.ListScrollFrame, "TOPLEFT", 48+(j-1)*256, -46*(i-1))
		end
	end
end

function AnvilMountJournal_OnEvent(self, event, ...)
	if ( event == "MOUNT_JOURNAL_USABILITY_CHANGED" or event == "COMPANION_LEARNED" or event == "COMPANION_UNLEARNED" or event == "COMPANION_UPDATE" ) then
		local companionType = ...;
		if ( not companionType or companionType == "MOUNT" ) then
			AnvilMountJournal_FullUpdate(self);
		end
	elseif ( event == "MOUNT_JOURNAL_SEARCH_UPDATED" ) then
		AnvilMountJournal_FullUpdate(self);
	elseif ( event == "UI_MODEL_SCENE_INFO_UPDATED" ) then
		if (self:IsVisible()) then
			AnvilMountJournal_UpdateMountDisplay(true);
		end
	end
end

function AnvilMountJournal_FullUpdate(self)
	if (self:IsVisible()) then
		AnvilMountJournal_UpdateMountList();
		if (not AnvilMountJournal.selectedSpellID) then
			AnvilMountJournal_Select(1);
		end
		AnvilMountJournal_UpdateMountDisplay();
	end
end

function AnvilMountJournal_OnShow(self)
	AnvilMountJournal_FullUpdate(self);
	SetPortraitToTexture(AnvilCollectionsJournalPortrait, "Interface\\Icons\\MountJournalPortrait");
end

function AnvilMountJournal_OnHide(self)
	C_MountJournal.ClearRecentFanfares();
end

function AnvilMountJournal_UpdateMountList()
	local scrollFrame = AnvilMountJournal.ListScrollFrame;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons = scrollFrame.buttons;

	local numMounts = C_MountJournal.GetNumMounts();
	AnvilMountJournal.numOwned = 0;
	local showMounts = true;
	local playerLevel = UnitLevel("player");
	if  ( numMounts < 1 ) then
		AnvilMountJournal.MountDisplay.NoMounts:Show();
		showMounts = false;
	else
		local mountIDs = C_MountJournal.GetMountIDs();
		for i, mountID in ipairs(mountIDs) do
			local _, _, _, _, _, _, _, _, _, hideOnChar, isCollected = C_MountJournal.GetMountInfoByID(mountID);
			if (isCollected and hideOnChar ~= true) then
				AnvilMountJournal.numOwned = AnvilMountJournal.numOwned + 1;
			end
		end
		AnvilMountJournal.MountDisplay.NoMounts:Hide();
	end

	local numDisplayedMounts = C_MountJournal.GetNumDisplayedMounts();

	for i=1, #buttons do
		local button = buttons[i];
		local displayIndex = i + offset;
		if ( displayIndex <= numDisplayedMounts and showMounts ) then
			local index = displayIndex;
			local creatureName, spellID, icon, active, isUsable, sourceType, isFavorite, isFactionSpecific, faction, isFiltered, isCollected, mountID = C_MountJournal.GetDisplayedMountInfo(index);
			local needsFanfare = C_MountJournal.NeedsFanfare(mountID);
			button.name:SetText(creatureName);
			button.icon:SetTexture(needsFanfare and COLLECTIONS_FANFARE_ICON or icon);
			button.new:SetShown(needsFanfare);
			button.newGlow:SetShown(needsFanfare);
			button.index = index;
			button.spellID = spellID;
			button.active = active;
			if (active) then
				button.DragButton.ActiveTexture:Show();
			else
				button.DragButton.ActiveTexture:Hide();
			end
			button:Show();
			if ( AnvilMountJournal.selectedSpellID == spellID ) then
				button.selected = true;
				button.selectedTexture:Show();
			else
				button.selected = false;
				button.selectedTexture:Hide();
			end
			button:SetEnabled(true);
			button.unusable:Hide();
			button.iconBorder:Hide();
			button.background:SetVertexColor(1, 1, 1, 1);
			if (isUsable or needsFanfare) then
				button.DragButton:SetEnabled(true);
				button.additionalText = nil;
				button.icon:SetDesaturated(false);
				button.icon:SetAlpha(1.0);
				button.name:SetFontObject("GameFontNormal");
			else
				if (isCollected) then
					button.unusable:Show();
					button.DragButton:SetEnabled(true);
					button.name:SetFontObject("GameFontNormal");
					button.icon:SetAlpha(0.75);
					button.additionalText = nil;
					button.background:SetVertexColor(1, 0, 0, 1);
				else
					button.icon:SetDesaturated(true);
					button.DragButton:SetEnabled(false);
					button.icon:SetAlpha(0.25);
					button.additionalText = nil;
					button.name:SetFontObject("GameFontDisable");
				end
			end
			if ( isFavorite ) then
				button.favorite:Show();
			else
				button.favorite:Hide();
			end
			if ( isFactionSpecific ) then
				button.factionIcon:SetAtlas(MOUNT_FACTION_TEXTURES[faction], true);
				button.factionIcon:Show();
			else
				button.factionIcon:Hide();
			end
			if ( button.showingTooltip ) then
				GameTooltip:SetMountBySpellID(button.spellID)
			end
		else
			button:Hide()
		end
	end
	local totalHeight = 46*(Anvil.db.global.collectionsWidth*ceil(numDisplayedMounts/Anvil.db.global.collectionsWidth)-11*(Anvil.db.global.collectionsWidth-1))+4
	HybridScrollFrame_Update(scrollFrame, totalHeight, scrollFrame:GetHeight());
	AnvilMountJournal.MountCount.Count:SetText(AnvilMountJournal.numOwned);
	if ( not showMounts ) then
		AnvilMountJournal.selectedSpellID = nil;
		AnvilMountJournal.selectedMountID = nil;
		AnvilMountJournal_UpdateMountDisplay();
		AnvilMountJournal.MountCount.Count:SetText(0);
	end
end

function AnvilMountJournalMountButton_ChooseFallbackMountToDisplay(mountID)
	local allCreatureDisplays = C_MountJournal.GetMountAllCreatureDisplayInfoByID(mountID);
	if allCreatureDisplays and #allCreatureDisplays > 0 then
		return allCreatureDisplays[math.random(1, #allCreatureDisplays)].creatureDisplayID;
	end
	return 0;
end

function AnvilMountJournal_UpdateMountDisplay(forceSceneChange)
	if ( AnvilMountJournal.selectedMountID ) then
		local creatureName, spellID, icon, active, isUsable, sourceType = C_MountJournal.GetMountInfoByID(AnvilMountJournal.selectedMountID);
		local needsFanfare = C_MountJournal.NeedsFanfare(AnvilMountJournal.selectedMountID);
		if ( AnvilMountJournal.MountDisplay.lastDisplayed ~= spellID or forceSceneChange ) then
			local creatureDisplayID, descriptionText, sourceText, isSelfMount, _, modelSceneID = C_MountJournal.GetMountInfoExtraByID(AnvilMountJournal.selectedMountID);
			if not creatureDisplayID then
				creatureDisplayID = AnvilMountJournalMountButton_ChooseFallbackMountToDisplay(AnvilMountJournal.selectedMountID);
			end
			AnvilMountJournal.MountDisplay.InfoButton.Name:SetText(creatureName);
			if needsFanfare then
				AnvilMountJournal.MountDisplay.InfoButton.New:Show();
				AnvilMountJournal.MountDisplay.InfoButton.NewGlow:Show();
				local offsetX = math.min(AnvilMountJournal.MountDisplay.InfoButton.Name:GetStringWidth(), AnvilMountJournal.MountDisplay.InfoButton.Name:GetWidth());
				AnvilMountJournal.MountDisplay.InfoButton.New:SetPoint("LEFT", AnvilMountJournal.MountDisplay.InfoButton.Name, "LEFT", offsetX + 8, 0);
				AnvilMountJournal.MountDisplay.InfoButton.Icon:SetTexture(COLLECTIONS_FANFARE_ICON);
			else
				AnvilMountJournal.MountDisplay.InfoButton.New:Hide();
				AnvilMountJournal.MountDisplay.InfoButton.NewGlow:Hide();
				AnvilMountJournal.MountDisplay.InfoButton.Icon:SetTexture(icon);
			end
			AnvilMountJournal.MountDisplay.InfoButton.Source:SetText(sourceText);
			AnvilMountJournal.MountDisplay.InfoButton.Lore:SetText(descriptionText)
			AnvilMountJournal.MountDisplay.lastDisplayed = spellID;
			AnvilMountJournal.MountDisplay.ModelScene:TransitionToModelSceneID(modelSceneID, CAMERA_TRANSITION_TYPE_IMMEDIATE, CAMERA_MODIFICATION_TYPE_MAINTAIN, forceSceneChange);
			AnvilMountJournal.MountDisplay.ModelScene:PrepareForFanfare(needsFanfare);
			local mountActor = AnvilMountJournal.MountDisplay.ModelScene:GetActorByTag("unwrapped");
			if mountActor then
				mountActor:SetModelByCreatureDisplayID(creatureDisplayID);
				if (isSelfMount) then
					mountActor:SetAnimationBlendOperation(LE_MODEL_BLEND_OPERATION_NONE);
					mountActor:SetAnimation(618);
				else
					mountActor:SetAnimationBlendOperation(LE_MODEL_BLEND_OPERATION_ANIM);
					mountActor:SetAnimation(0);
				end
			end
		end
		AnvilMountJournal.MountDisplay.ModelScene:Show();
		AnvilMountJournal.MountDisplay.InfoButton:Show();
		AnvilMountJournal.MountDisplay.NoMounts:Hide();
		if ( needsFanfare ) then
			AnvilMountJournal.AnvilMountButton:SetText(UNWRAP)
			AnvilMountJournal.AnvilMountButton:Enable();
		elseif ( active ) then
			AnvilMountJournal.AnvilMountButton:SetText(BINDING_NAME_DISMOUNT);
			AnvilMountJournal.AnvilMountButton:SetEnabled(isUsable);
		else
			AnvilMountJournal.AnvilMountButton:SetText(MOUNT);
			AnvilMountJournal.AnvilMountButton:SetEnabled(isUsable);
		end
	else
		AnvilMountJournal.MountDisplay.InfoButton:Hide();
		AnvilMountJournal.MountDisplay.ModelScene:Hide();
		AnvilMountJournal.MountDisplay.NoMounts:Show();
		AnvilMountJournalMountButton:SetEnabled(false);
	end
end

function AnvilMountJournal_Select(index)
	local creatureName, spellID, icon, active, _, _, _, _, _, _, _, mountID = C_MountJournal.GetDisplayedMountInfo(index);
	AnvilMountJournal.selectedSpellID = spellID;
	AnvilMountJournal.selectedMountID = mountID;
	AnvilMountJournal_HideMountDropdown();
	AnvilMountJournal_UpdateMountList();
	AnvilMountJournal_UpdateMountDisplay();
end

function AnvilMountJournalMountButton_UseMount(mountID)
	local active = select(4,C_MountJournal.GetMountInfoByID(mountID))
	if ( active ) then
		C_MountJournal.Dismiss();
	elseif ( C_MountJournal.NeedsFanfare(mountID) ) then
		local function OnFinishedCallback()
			C_MountJournal.ClearFanfare(mountID);
			AnvilMountJournal_HideMountDropdown();
			AnvilMountJournal_UpdateMountList();
			AnvilMountJournal_UpdateMountDisplay();
		end
		AnvilMountJournal.MountDisplay.ModelScene:StartUnwrapAnimation(OnFinishedCallback);
	else
		C_MountJournal.SummonByID(mountID);
	end
end

function AnvilMountJournalMountButton_OnClick()
	if AnvilMountJournal.selectedMountID then
		AnvilMountJournalMountButton_UseMount(AnvilMountJournal.selectedMountID);
	end
end

function AnvilMountListDragButton_OnClick(self, button)
	local parent = self:GetParent();
	if ( button ~= "LeftButton" ) then
		local isCollected = select(11,C_MountJournal.GetDisplayedMountInfo(parent.index))
		if isCollected then
			AnvilMountJournal_ShowMountDropdown(parent.index, self, 0, 0);
		end
	elseif ( IsModifiedClick("CHATLINK") ) then
		local id = parent.spellID;
		if ( MacroFrame and MacroFrame:IsShown() ) then
			local spellName = GetSpellInfo(id);
			ChatEdit_InsertLink(spellName);
		else
			local spellLink = GetSpellLink(id)
			ChatEdit_InsertLink(spellLink);
		end
	else
		C_MountJournal.Pickup(parent.index);
	end
end

function AnvilMountListItem_OnClick(self, button)
	if ( button ~= "LeftButton" ) then
		local isCollected = selected(11,C_MountJournal.GetDisplayedMountInfo(self.index))
		if isCollected then
			AnvilMountJournal_ShowMountDropdown(self.index, self, 0, 0);
		end
	elseif ( IsModifiedClick("CHATLINK") ) then
		local id = self.spellID;
		if ( MacroFrame and MacroFrame:IsShown() ) then
			local spellName = GetSpellInfo(id);
			ChatEdit_InsertLink(spellName);
		else
			local spellLink = GetSpellLink(id)
			ChatEdit_InsertLink(spellLink);
		end
	elseif ( self.spellID ~= AnvilMountJournal.selectedSpellID ) then
		AnvilMountJournal_Select(self.index);
	end
end

function AnvilMountJournal_OnSearchTextChanged(self)
	SearchBoxTemplate_OnTextChanged(self);
	C_MountJournal.SetSearch(self:GetText());
end

function AnvilMountJournal_ClearSearch()
	AnvilMountJournal.searchBox:SetText("");
end

function AnvilMountJournalFilterDropDown_OnLoad(self)
	UIDropDownMenu_Initialize(self, AnvilMountJournalFilterDropDown_Initialize, "MENU");
end

function AnvilMountJournalFilterDropDown_Initialize(self, level)
	local info = UIDropDownMenu_CreateInfo();
	info.keepShownOnClick = true;

	if level == 1 then
		info.text = COLLECTED
		info.func = function(_, _, _, value)
						C_MountJournal.SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED,value);
					end
		info.checked = C_MountJournal.GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED);
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = NOT_COLLECTED
		info.func = function(_, _, _, value)
						C_MountJournal.SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_NOT_COLLECTED,value);
					end
		info.checked = C_MountJournal.GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_NOT_COLLECTED);
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = MOUNT_JOURNAL_FILTER_UNUSABLE
		info.func = function(_, _, _, value)
						C_MountJournal.SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_UNUSABLE, value);
					end
		info.checked = C_MountJournal.GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_UNUSABLE);
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.checked = 	nil;
		info.isNotRadio = nil;
		info.func =  nil;
		info.hasArrow = true;
		info.notCheckable = true;

		info.text = SOURCES;
		info.value = 1;
		UIDropDownMenu_AddButton(info, level)
	else
		info.hasArrow = false;
		info.isNotRadio = true;
		info.notCheckable = true;

		info.text = CHECK_ALL
		info.func = function()
			C_MountJournal.SetAllSourceFilters(true);
			UIDropDownMenu_Refresh(AnvilMountJournalFilterDropDown, 1, 2);
		end
		UIDropDownMenu_AddButton(info, level)

		info.text = UNCHECK_ALL
		info.func = function()
			C_MountJournal.SetAllSourceFilters(false);
			UIDropDownMenu_Refresh(AnvilMountJournalFilterDropDown, 1, 2);
		end
		UIDropDownMenu_AddButton(info, level)
		info.notCheckable = false;
		local numSources = C_PetJournal.GetNumPetSources();
		for i=1,numSources do
			if C_MountJournal.IsValidSourceFilter(i) then
				info.text = _G["BATTLE_PET_SOURCE_"..i];
				info.func = function(_, _, _, value)
					C_MountJournal.SetSourceFilter(i,value);
				end
				info.checked = function() return C_MountJournal.IsSourceChecked(i) end;
				UIDropDownMenu_AddButton(info, level);
			end
		end
	end
end

function AnvilMountJournalSummonRandomFavoriteButton_OnLoad(self)
	self.spellID = SUMMON_RANDOM_FAVORITE_MOUNT_SPELL;
	local spellName, spellSubname, spellIcon = GetSpellInfo(self.spellID);
	self.texture:SetTexture(spellIcon);
	self.spellname:SetText(MOUNT_JOURNAL_SUMMON_RANDOM_FAVORITE_MOUNT);
	self:RegisterForDrag("LeftButton");
end

function AnvilMountJournalSummonRandomFavoriteButton_OnClick(self)
	C_MountJournal.SummonByID(0);
end

function AnvilMountJournalSummonRandomFavoriteButton_OnDragStart(self)
	C_MountJournal.Pickup(0);
end

function AnvilMountJournalSummonRandomFavoriteButton_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetMountBySpellID(self.spellID);
end

function AnvilMountOptionsMenu_Init(self, level)
	if not AnvilMountJournal.menuMountIndex then
		return;
	end

	local info = UIDropDownMenu_CreateInfo();
	info.notCheckable = true;
	local active = select(4, C_MountJournal.GetMountInfoByID(AnvilMountJournal.menuMountID));
	local needsFanfare = C_MountJournal.NeedsFanfare(AnvilMountJournal.menuMountID);
	if (needsFanfare) then
		info.text = UNWRAP;
	elseif ( active ) then
		info.text = BINDING_NAME_DISMOUNT;
	else
		info.text = MOUNT;
		info.disabled = not AnvilMountJournal.menuIsUsable;
	end
	info.func = function()
		if needsFanfare then
			AnvilMountJournal_Select(AnvilMountJournal.menuMountIndex);
		end
		AnvilMountJournalMountButton_UseMount(AnvilMountJournal.menuMountID);
	end;

	UIDropDownMenu_AddButton(info, level);

	if not needsFanfare then
		info.disabled = nil;
		local canFavorite = false;
		local isFavorite = false;
		if (AnvilMountJournal.menuMountIndex) then
			 isFavorite, canFavorite = C_MountJournal.GetIsFavorite(AnvilMountJournal.menuMountIndex);
		end

		if (isFavorite) then
			info.text = BATTLE_PET_UNFAVORITE;
			info.func = function()
				C_MountJournal.SetIsFavorite(AnvilMountJournal.menuMountIndex, false);
			end
		else
			info.text = BATTLE_PET_FAVORITE;
			info.func = function()
				C_MountJournal.SetIsFavorite(AnvilMountJournal.menuMountIndex, true);
			end
		end

		if (canFavorite) then
			info.disabled = false;
		else
			info.disabled = true;
		end

		UIDropDownMenu_AddButton(info, level);
	end

	info.disabled = nil;
	info.text = CANCEL
	info.func = nil
	UIDropDownMenu_AddButton(info, level)
end

function AnvilMountJournal_ShowMountDropdown(index, anchorTo, offsetX, offsetY)
	if (index) then
		AnvilMountJournal.menuMountIndex = index;
		AnvilMountJournal.menuMountID = select(12, C_MountJournal.GetDisplayedMountInfo(AnvilMountJournal.menuMountIndex));
		local active, isUsable = select(4, C_MountJournal.GetDisplayedMountInfo(index));
		AnvilMountJournal.active = active;
		AnvilMountJournal.menuIsUsable = isUsable;
	else
		return;
	end
	ToggleDropDownMenu(1, nil, AnvilMountJournal.mountOptionsMenu, anchorTo, offsetX, offsetY);
	PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
end

function AnvilMountJournal_HideMountDropdown()
	if (UIDropDownMenu_GetCurrentDropDown() == AnvilMountJournal.mountOptionsMenu) then
		HideDropDownMenu(1);
	end
end
