AnvilPets = Anvil:NewModule("AnvilPets", "AceConsole-3.0", "AceEvent-3.0")
local _G = _G

local COMPANION_BUTTON_HEIGHT = 46;
local MAX_ACTIVE_PETS = 3;
local NUM_PET_ABILITIES = 6;
local MAX_PET_LEVEL = 25;
local HEAL_PET_SPELL = 125439;
local AnvilStaticPopupDialogs = {}

local UNLOCK_REQUIREMENTS = {
	[1] = {requirement = "SPELL", id = "119467"},
	[2] = {requirement = "ACHIEVEMENT", id = "7433"},
	[3] = {requirement = "ACHIEVEMENT", id = "6566"}
};

AnvilStaticPopupDialogs["BATTLE_PET_RENAME"] = {
	text = PET_RENAME_LABEL,
	button1 = ACCEPT,
	button3 = PET_RENAME_DEFAULT_LABEL,
	button2 = CANCEL,
	hasEditBox = 1,
	maxLetters = 16,
	OnAccept = function(self)
		local text = self.editBox:GetText();
		C_PetJournal.SetCustomName(self.data, text);
		AnvilPetJournal_UpdateAll();
	end,
	OnAlt = function(self)
		C_PetJournal.SetCustomName(self.data, "");
		AnvilPetJournal_UpdateAll();
	end,
	EditBoxOnEnterPressed = function(self)
		local parent = self:GetParent();
		local text = parent.editBox:GetText();
		C_PetJournal.SetCustomName(parent.data, text);
		AnvilPetJournal_UpdateAll();
		parent:Hide();
	end,
	OnShow = function(self)
		self.editBox:SetFocus();
	end,
	OnHide = function(self)
		ChatEdit_FocusActiveWindow();
		self.editBox:SetText("");
	end,
	timeout = 0,
	exclusive = 1,
	hideOnEscape = 1
};

AnvilStaticPopupDialogs["BATTLE_PET_PUT_IN_CAGE"] = {
	text = PET_PUT_IN_CAGE_LABEL,
	button1 = OKAY,
	button2 = CANCEL,
	maxLetters = 30,
	OnAccept = function(self)
		AnvilPetJournal_SetPendingCage(self.data);
		C_PetJournal.CagePetByID(self.data);
		if (AnvilPetJournalPetCard.petID == self.data) then
			AnvilPetJournal_ShowPetCard(1);
		end
	end,
	timeout = 0,
	exclusive = 1,
	hideOnEscape = 1
};

AnvilStaticPopupDialogs["BATTLE_PET_RELEASE"] = {
	text = "\n\n" .. PET_RELEASE_LABEL .. "\n\n",
	button1 = OKAY,
	button2 = CANCEL,
	maxLetters = 30,
	OnAccept = function(self)
		C_PetJournal.ReleasePetByID(self.data);
		if (AnvilPetJournalPetCard.petID == self.data) then
			AnvilPetJournal_ShowPetCard(1);
		end
	end,
	timeout = 0,
	exclusive = 1,
	hideOnEscape = 1,
	showAlert = 1,
};

function AnvilPets:OnEnable()
	if not Anvil.db.global.collections then
		return
	else
		AnvilPetJournal:RegisterEvent("UNIT_PORTRAIT_UPDATE");
		AnvilPetJournal:RegisterEvent("PET_JOURNAL_LIST_UPDATE");
		AnvilPetJournal:RegisterEvent("PET_JOURNAL_PET_DELETED");
		AnvilPetJournal:RegisterEvent("PET_JOURNAL_PETS_HEALED");
		AnvilPetJournal:RegisterEvent("PET_JOURNAL_CAGE_FAILED");
		AnvilPetJournal:RegisterEvent("BATTLE_PET_CURSOR_CLEAR");
		AnvilPetJournal:RegisterEvent("COMPANION_UPDATE");
		AnvilPetJournal:RegisterEvent("PET_BATTLE_LEVEL_CHANGED");
		AnvilPetJournal:RegisterEvent("PET_BATTLE_QUEUE_STATUS");
		AnvilPetJournal:RegisterEvent("UI_MODEL_SCENE_INFO_UPDATED");
		AnvilPetJournal.listScroll.update = AnvilPetJournal_UpdatePetList;
		AnvilPetJournal.listScroll.scrollBar.doNotHide = true;
		HybridScrollFrame_CreateButtons(AnvilPetJournal.listScroll, "AnvilCompanionListButtonTemplate", 44, 0);
		self:ResizePetJournal()
		UIDropDownMenu_Initialize(AnvilPetJournal.petOptionsMenu, AnvilPetOptionsMenu_Init, "MENU");
	end
end

function AnvilPets:OnDisable()
	AnvilPetJournal:UnregisterAllEvents()
end

function AnvilPets:ResizePetJournal()
	AnvilPetJournalListScrollFrame.stepSize = Anvil.db.global.collectionsWidth*46
	if AnvilPetJournalListScrollFrame.buttons then
		wipe(AnvilPetJournalListScrollFrame.buttons)
	else
		AnvilPetJournalListScrollFrame.buttons = {}
	end
	for i=1, 55 do
		if i<=11*Anvil.db.global.collectionsWidth then
			if not _G["AnvilPetJournalListScrollFrameButton"..i] then
				_G["AnvilPetJournalListScrollFrameButton"..i] = CreateFrame("Button", "AnvilPetJournalListScrollFrameButton" .. i, AnvilPetJournalListScrollFrameScrollChild, "AnvilCompanionListButtonTemplate")
			end
			tinsert(AnvilPetJournalListScrollFrame.buttons, _G["AnvilPetJournalListScrollFrameButton"..i])
		elseif _G["AnvilPetJournalListScrollFrameButton"..i] then
			_G["AnvilPetJournalListScrollFrameButton"..i]:Hide()
		end
	end
	AnvilPetJournal_UpdatePetList()
	for i=1, 11 do
		for j=1, Anvil.db.global.collectionsWidth do
			_G["AnvilPetJournalListScrollFrameButton"..(i-1)*Anvil.db.global.collectionsWidth+j]:SetPoint("TOPLEFT", AnvilPetJournalListScrollFrame, "TOPLEFT", 48+(j-1)*256, -46*(i-1))
		end
	end
end

function AnvilPetJournal_OnShow(self)
	AnvilPetJournal_UpdatePetList();
	AnvilPetJournal_UpdatePetLoadOut();
	AnvilPetJournal_UpdatePetCard(AnvilPetJournalPetCard);

	self:RegisterEvent("ACHIEVEMENT_EARNED");
	AnvilPetJournal.AchievementStatus.SumText:SetText(GetCategoryAchievementPoints(PET_ACHIEVEMENT_CATEGORY, true));

	if ( not GetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_PET_JOURNAL) ) then
		local helpPlate = AnvilPetJournal_HelpPlate;
		if ( helpPlate and not HelpPlate_IsShowing(helpPlate) ) then
			HelpPlate_ShowTutorialPrompt( helpPlate, AnvilPetJournal.MainHelpButton );
			SetCVarBitfield( "closedInfoFrames", LE_FRAME_TUTORIAL_PET_JOURNAL, true );
		end
	end
	if not AnvilPetJournalPetCard.petIndex then
		AnvilPetJournal_ShowPetCard(1);
	end
	SetPortraitToTexture(AnvilCollectionsJournalPortrait, "Interface\\Icons\\PetJournalPortrait");
end

function AnvilPetJournal_OnHide(self)
	self:UnregisterEvent("ACHIEVEMENT_EARNED");
	AnvilPetJournal.SpellSelect:Hide();
	HelpPlate_Hide();
	C_PetJournal.ClearRecentFanfares();
end

function AnvilPetJournal_OnEvent(self, event, ...)
	if event == "PET_BATTLE_LEVEL_CHANGED" then
		AnvilPetJournal_UpdatePetLoadOut();
	elseif event == "PET_JOURNAL_PET_DELETED" then
		local petID = ...;
		if (AnvilPetJournal_IsPendingCage(petID)) then
			AnvilPetJournal_ClearPendingCage();
		end
		AnvilPetJournal_UpdatePetList();
		AnvilPetJournal_UpdatePetLoadOut();
		if(AnvilPetJournalPetCard.petID == petID) then
			AnvilPetJournal_ShowPetCard(1);
		end
		AnvilPetJournal_FindPetCardIndex();
		AnvilPetJournal_UpdatePetCard(AnvilPetJournalPetCard);
		AnvilPetJournal_HidePetDropdown();
	elseif event == "PET_JOURNAL_CAGE_FAILED" then
		AnvilPetJournal_ClearPendingCage();
	elseif event == "PET_JOURNAL_PETS_HEALED" then
		AnvilPetJournal_UpdatePetLoadOut();
	elseif event == "PET_JOURNAL_LIST_UPDATE" then
		AnvilPetJournal_FindPetCardIndex();
		AnvilPetJournal_UpdatePetList();
		AnvilPetJournal_UpdatePetLoadOut();
		AnvilPetJournal_UpdatePetCard(AnvilPetJournalPetCard);
		AnvilPetJournal_HidePetDropdown();
	elseif event == "BATTLE_PET_CURSOR_CLEAR" then
		AnvilPetJournal.Loadout.Pet1.setButton:Hide();
		AnvilPetJournal.Loadout.Pet2.setButton:Hide();
		AnvilPetJournal.Loadout.Pet3.setButton:Hide();
	elseif event == "COMPANION_UPDATE" then
		local companionType = ...;
		if companionType == "CRITTER" then
			AnvilPetJournal_UpdatePetList();
			AnvilPetJournal_UpdateSummonButtonState();
		end
	elseif event == "ACHIEVEMENT_EARNED" then
		AnvilPetJournal.AchievementStatus.SumText:SetText(GetCategoryAchievementPoints(PET_ACHIEVEMENT_CATEGORY, true));
	elseif event == "PET_BATTLE_QUEUE_STATUS" then
		AnvilPetJournal_UpdatePetLoadOut();
	elseif event == "UI_MODEL_SCENE_INFO_UPDATED" then
		if (self:IsVisible()) then
			AnvilPetJournal_UpdatePetCard(AnvilPetJournalPetCard, true);
			AnvilPetJournal_UpdatePetLoadOut(true);
		end
	end
end

function AnvilPetJournal_SelectSpecies(self, targetSpeciesID)
	local numPets = C_PetJournal.GetNumPets();
	local petIndex = nil;
	for i = 1,numPets do
		local petID, speciesID, owned = C_PetJournal.GetPetInfoByIndex(i);
		if (speciesID == targetSpeciesID) then
			petIndex = i;
			break;
		end
	end

	if ( petIndex ) then
		AnvilPetJournalPetList_UpdateScrollPos(self.listScroll, petIndex);
	end
	AnvilPetJournal_ShowPetCardBySpeciesID(targetSpeciesID);
end

function AnvilPetJournal_SelectPet(self, targetPetID)
	local numPets = C_PetJournal.GetNumPets();
	local petIndex = nil;
	for i = 1,numPets do
		local petID, speciesID, owned = C_PetJournal.GetPetInfoByIndex(i);
		if (petID == targetPetID) then
			petIndex = i;
			break;
		end
	end

	if ( petIndex ) then
		AnvilPetJournalPetList_UpdateScrollPos(self.listScroll, petIndex);
	end
	AnvilPetJournal_ShowPetCardByID(targetPetID);
end

function AnvilPetJournal_UpdateSummonButtonState()
	local petID = AnvilPetJournalPetCard.petID;
	local hasPetID = petID ~= nil;
	local needsFanfare = hasPetID and C_PetJournal.PetNeedsFanfare(petID);

	AnvilPetJournal.SummonButton:SetEnabled(hasPetID and (C_PetJournal.PetIsSummonable(petID) or needsFanfare));

	if hasPetID and petID == C_PetJournal.GetSummonedPetGUID() then
		AnvilPetJournal.SummonButton:SetText(PET_DISMISS);
	elseif needsFanfare then
		AnvilPetJournal.SummonButton:SetText(UNWRAP);
	else
		AnvilPetJournal.SummonButton:SetText(BATTLE_PET_SUMMON);
	end

	if (GameTooltip:GetOwner() == AnvilPetJournal.SummonButton) then
		AnvilPetJournalSummonButton_OnEnter(AnvilPetJournal.SummonButton);
	end
end


function AnvilPetJournal_ShowPetSelect(self)
	local slotFrame = self:GetParent();
	local abilities = slotFrame.abilities;
	local slotIndex = slotFrame:GetID();

	local abilityIndex = self:GetID();
	local spellIndex1 = abilityIndex;
	local spellIndex2 = spellIndex1 + 3;

	local speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType = C_PetJournal.GetPetInfoByPetID(slotFrame.petID);

	if AnvilPetJournal.SpellSelect:IsShown() then
		if AnvilPetJournal.SpellSelect.slotIndex == slotIndex and AnvilPetJournal.SpellSelect.abilityIndex == abilityIndex then
			AnvilPetJournal.SpellSelect:Hide();
			self.selected:Hide();
			return;
		else
			AnvilPetJournal.Loadout["Pet"..AnvilPetJournal.SpellSelect.slotIndex]["spell"..AnvilPetJournal.SpellSelect.abilityIndex].selected:Hide();
		end
	end
	self.selected:Show();
	AnvilPetJournal.SpellSelect.slotIndex = slotIndex;
	AnvilPetJournal.SpellSelect.abilityIndex = abilityIndex;
	AnvilPetJournal.SpellSelect:SetFrameLevel(AnvilPetJournalLoadoutBorder:GetFrameLevel()+1);
	AnvilPetJournal_HideAbilityTooltip();

	local name, icon, petType, requiredLevel;
	if (abilities[spellIndex1]) then
		name, icon, petType = C_PetJournal.GetPetAbilityInfo(abilities[spellIndex1]);
		requiredLevel = AnvilPetJournalLoadout_GetRequiredLevel(slotFrame, abilities[spellIndex1]);
		AnvilPetJournal.SpellSelect.Spell1:SetEnabled(requiredLevel <= level);
	else
		name = "";
		icon = "";
		petType = "";
		requiredLevel = 0;
		AnvilPetJournal.SpellSelect.Spell1:SetEnabled(false);
	end

	if ( requiredLevel > level ) then
		AnvilPetJournal.SpellSelect.Spell1.additionalText = format(PET_ABILITY_REQUIRES_LEVEL, requiredLevel);
	else
		AnvilPetJournal.SpellSelect.Spell1.additionalText = nil;
	end
	AnvilPetJournal.SpellSelect.Spell1.icon:SetTexture(icon);
	AnvilPetJournal.SpellSelect.Spell1.icon:SetDesaturated(requiredLevel > level);
	AnvilPetJournal.SpellSelect.Spell1.BlackCover:SetShown(requiredLevel > level);
	AnvilPetJournal.SpellSelect.Spell1.LevelRequirement:SetShown(requiredLevel > level);
	AnvilPetJournal.SpellSelect.Spell1.LevelRequirement:SetText(requiredLevel);
	AnvilPetJournal.SpellSelect.Spell1.slotIndex = slotIndex;
	AnvilPetJournal.SpellSelect.Spell1.abilityIndex = abilityIndex;
	AnvilPetJournal.SpellSelect.Spell1.abilityID = abilities[spellIndex1];
	AnvilPetJournal.SpellSelect.Spell1.petID = slotFrame.petID;
	AnvilPetJournal.SpellSelect.Spell1.speciesID = slotFrame.speciesID;

	if (abilities[spellIndex2]) then
		name, icon, petType = C_PetJournal.GetPetAbilityInfo(abilities[spellIndex2]);
		requiredLevel = AnvilPetJournalLoadout_GetRequiredLevel(slotFrame, abilities[spellIndex2]);
		AnvilPetJournal.SpellSelect.Spell2:SetEnabled(requiredLevel <= level);
	else
		name = "";
		icon = "";
		petType = "";
		requiredLevel = 0;
		AnvilPetJournal.SpellSelect.Spell2:SetEnabled(false);
	end

	if ( requiredLevel > level ) then
		AnvilPetJournal.SpellSelect.Spell2.additionalText = format(PET_ABILITY_REQUIRES_LEVEL, requiredLevel);
	else
		AnvilPetJournal.SpellSelect.Spell2.additionalText = nil;
	end
	AnvilPetJournal.SpellSelect.Spell2.icon:SetTexture(icon);
	AnvilPetJournal.SpellSelect.Spell2.BlackCover:SetShown(requiredLevel > level);
	AnvilPetJournal.SpellSelect.Spell2.icon:SetDesaturated(requiredLevel > level);
	AnvilPetJournal.SpellSelect.Spell2.LevelRequirement:SetShown(requiredLevel > level);
	AnvilPetJournal.SpellSelect.Spell2.LevelRequirement:SetText(requiredLevel);
	AnvilPetJournal.SpellSelect.Spell2.slotIndex = slotIndex;
	AnvilPetJournal.SpellSelect.Spell2.abilityIndex = abilityIndex;
	AnvilPetJournal.SpellSelect.Spell2.abilityID = abilities[spellIndex2];
	AnvilPetJournal.SpellSelect.Spell2.petID = slotFrame.petID;
	AnvilPetJournal.SpellSelect.Spell2.speciesID = slotFrame.speciesID;

	AnvilPetJournal.SpellSelect.Spell1:SetChecked(self.abilityID == abilities[spellIndex1]);
	AnvilPetJournal.SpellSelect.Spell2:SetChecked(self.abilityID == abilities[spellIndex2]);

	AnvilPetJournal.SpellSelect:SetPoint("TOP", self, "BOTTOM", 0, 0);
	AnvilPetJournal.SpellSelect:Show();
end

function AnvilPetJournal_UpdatePetLoadOut(forceSceneChange)
	AnvilPetJournal_UpdateFindBattleButton();
	AnvilPetJournal.SpellSelect:Hide();
	for i=1,MAX_ACTIVE_PETS do
		local loadoutPlate = AnvilPetJournal.Loadout["Pet"..i];
		local petID, ability1ID, ability2ID, ability3ID, locked = C_PetJournal.GetPetLoadOutInfo(i);

		if ( not C_PetJournal.IsJournalUnlocked() ) then
			loadoutPlate.ReadOnlyFrame:Show();
			loadoutPlate.ReadOnlyFrame.LockIcon.tooltip = PET_JOURNAL_READONLY_TEXT;
		elseif ( C_PetBattles.GetPVPMatchmakingInfo() ) then
			loadoutPlate.ReadOnlyFrame:Show();
			loadoutPlate.ReadOnlyFrame.LockIcon.tooltip = ERR_PETBATTLE_QUEUE_QUEUED;
		else
			loadoutPlate.ReadOnlyFrame:Hide();
		end

		if (locked) then
			loadoutPlate.name:Hide();
			loadoutPlate.subName:Hide();
			loadoutPlate.level:Hide();
			loadoutPlate.levelBG:Hide();
			loadoutPlate.icon:Hide();
			loadoutPlate.qualityBorder:Hide();
			loadoutPlate.favorite:Hide()
			loadoutPlate.modelScene:Hide();
			loadoutPlate.xpBar:Hide();
			loadoutPlate.healthFrame:Hide();
			loadoutPlate.spell1:Hide();
			loadoutPlate.spell2:Hide();
			loadoutPlate.spell3:Hide();
			loadoutPlate.iconBorder:Hide();
			loadoutPlate.emptyslot:Hide();
			loadoutPlate.isDead:Hide();
			loadoutPlate.dragButton:Hide();
			loadoutPlate.requirement:SetShown(UNLOCK_REQUIREMENTS[i].id);
			if (UNLOCK_REQUIREMENTS[i].requirement == "ACHIEVEMENT" and UNLOCK_REQUIREMENTS[i].id) then
				loadoutPlate.requirement.str:SetText(GetAchievementLink(UNLOCK_REQUIREMENTS[i].id));
				loadoutPlate.requirement.achievementID = UNLOCK_REQUIREMENTS[i].id;
			elseif (UNLOCK_REQUIREMENTS[i].requirement == "SPELL" and UNLOCK_REQUIREMENTS[i].id) then
				loadoutPlate.requirement.str:SetText(GetSpellLink(UNLOCK_REQUIREMENTS[i].id));
				loadoutPlate.requirement.spellID = UNLOCK_REQUIREMENTS[i].id;
			end
			loadoutPlate.helpFrame.text:SetText(_G["BATTLE_PET_UNLOCK_HELP_"..i]);
			loadoutPlate.helpFrame:Show();
			loadoutPlate.petTypeIcon:Hide();
			loadoutPlate.petID = nil;
		elseif (petID == nil) then
			loadoutPlate.name:Hide();
			loadoutPlate.subName:Hide();
			loadoutPlate.level:Hide();
			loadoutPlate.levelBG:Hide();
			loadoutPlate.icon:Hide();
			loadoutPlate.qualityBorder:Hide();
			loadoutPlate.favorite:Hide()
			loadoutPlate.modelScene:Hide();
			loadoutPlate.xpBar:Hide();
			loadoutPlate.healthFrame:Hide();
			loadoutPlate.spell1:Hide();
			loadoutPlate.spell2:Hide();
			loadoutPlate.spell3:Hide();
			loadoutPlate.iconBorder:Hide();
			loadoutPlate.helpFrame:Hide();
			loadoutPlate.requirement:Hide();
			loadoutPlate.emptyslot:Show();
			loadoutPlate.emptyslot.slot:SetText(format(BATTLE_PET_SLOT, i));
			loadoutPlate.dragButton:Show();
			loadoutPlate.isDead:Hide();
			loadoutPlate.petTypeIcon:Hide();
			loadoutPlate.petID = nil;
		else
			local speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType = C_PetJournal.GetPetInfoByPetID(petID);
			C_PetJournal.GetPetAbilityList(speciesID, loadoutPlate.abilities, loadoutPlate.abilityLevels);

			local numUsableAbilities = 0;
			for j=1, #loadoutPlate.abilityLevels do
				if ( loadoutPlate.abilityLevels[j] and level >= loadoutPlate.abilityLevels[j] ) then
					numUsableAbilities = numUsableAbilities + 1;
				end
			end

			if ( ability1ID == 0 and loadoutPlate.abilities[1] ) then
				ability1ID = loadoutPlate.abilities[1];
				C_PetJournal.SetAbility(i, 1, ability1ID);
			end
			if ( ability2ID == 0 and loadoutPlate.abilities[2] ) then
				ability2ID = loadoutPlate.abilities[2];
				C_PetJournal.SetAbility(i, 2, ability2ID);
			end
			if ( ability3ID == 0 and loadoutPlate.abilities[3] ) then
				ability3ID = loadoutPlate.abilities[3];
				C_PetJournal.SetAbility(i, 3, ability3ID);
			end
			loadoutPlate.name:Show();
			loadoutPlate.subName:Show();
			loadoutPlate.level:Show();
			loadoutPlate.levelBG:Show();
			loadoutPlate.icon:Show();
			loadoutPlate.iconBorder:Show();
			loadoutPlate.spell1:Show();
			loadoutPlate.spell2:Show();
			loadoutPlate.spell3:Show();

			if (isFavorite) then
				loadoutPlate.favorite:Show()
			else
				loadoutPlate.favorite:Hide()
			end

			if customName then
				loadoutPlate.name:SetText(customName);
				loadoutPlate.name:SetHeight(12);
				loadoutPlate.subName:Show();
				loadoutPlate.subName:SetText(name);
			else
				loadoutPlate.name:SetText(name);
				loadoutPlate.name:SetHeight(28);
				loadoutPlate.subName:Hide();
			end
			loadoutPlate.level:SetText(level);
			loadoutPlate.icon:SetTexture(icon);

			loadoutPlate.petTypeIcon:Show();
			loadoutPlate.petTypeIcon:SetTexture(GetPetTypeTexture(petType));
			loadoutPlate.petID = petID;
			loadoutPlate.speciesID = speciesID;
			if(level < MAX_PET_LEVEL) then
				loadoutPlate.xpBar:Show();
			else
				loadoutPlate.xpBar:Hide();
			end

			loadoutPlate.xpBar:SetMinMaxValues(0, maxXp);
			loadoutPlate.xpBar:SetValue(xp);
			local display = GetCVar("statusTextDisplay")
			if (display == "BOTH") then
				loadoutPlate.xpBar.rankText:SetFormattedText(PET_BATTLE_CURRENT_XP_FORMAT_BOTH, xp, maxXp, xp/maxXp*100);
			elseif (display == "PERCENTAGE") then
				loadoutPlate.xpBar.rankText:SetFormattedText(PET_BATTLE_CURRENT_XP_FORMAT_PERCENT, xp/maxXp*100);
			else
				loadoutPlate.xpBar.rankText:SetFormattedText(PET_BATTLE_CURRENT_XP_FORMAT_VERBOSE, xp, maxXp);
			end
			loadoutPlate.xpBar.tooltip = format(PET_BATTLE_CURRENT_XP_FORMAT_TOOLTIP, xp, maxXp, xp/maxXp*100);

			local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);
			loadoutPlate.healthFrame.healthValue:SetFormattedText(PET_BATTLE_CURRENT_HEALTH_FORMAT, health, maxHealth);
			loadoutPlate.healthFrame.healthBar:SetMinMaxValues(0, maxHealth);
			loadoutPlate.healthFrame.healthBar:SetValue(health);
			loadoutPlate.healthFrame:Show();

			loadoutPlate.qualityBorder:SetVertexColor(ITEM_QUALITY_COLORS[rarity-1].r, ITEM_QUALITY_COLORS[rarity-1].g, ITEM_QUALITY_COLORS[rarity-1].b);

			loadoutPlate.isDead:SetShown(health <= 0);

			loadoutPlate.modelScene:Show();
			local modelChanged = false;
			if ( displayID ~= loadoutPlate.displayID or forceSceneChange ) then
				loadoutPlate.displayID = displayID;

				local cardModelSceneID, loadoutModelSceneID = C_PetJournal.GetPetModelSceneInfoBySpeciesID(speciesID);

				loadoutPlate.modelScene:TransitionToModelSceneID(loadoutModelSceneID, CAMERA_TRANSITION_TYPE_IMMEDIATE, CAMERA_MODIFICATION_TYPE_DISCARD, forceSceneChange);

				local battlePetActor = loadoutPlate.modelScene:GetActorByTag("pet");
				if ( battlePetActor ) then
					battlePetActor:SetModelByCreatureDisplayID(displayID);
					battlePetActor:SetAnimationBlendOperation(LE_MODEL_BLEND_OPERATION_NONE);
				end

				modelChanged = true;
			end
			local isDead = health <= 0;
			if ( modelChanged or isDead ~= loadoutPlate.modelScene.wasDead ) then
				local battlePetActor = loadoutPlate.modelScene:GetActorByTag("pet");
				if ( battlePetActor ) then
					if ( isDead ) then
						battlePetActor:SetAnimation(6, -1);
					else
						battlePetActor:SetAnimation(0, -1);
					end
				end
				loadoutPlate.modelScene.wasDead = isDead;
			end


			AnvilPetJournal_UpdatePetAbility(loadoutPlate.spell1, ability1ID, petID);
			AnvilPetJournal_UpdatePetAbility(loadoutPlate.spell2, ability2ID, petID);
			AnvilPetJournal_UpdatePetAbility(loadoutPlate.spell3, ability3ID, petID);

			loadoutPlate.spell1.enabled = true;
			loadoutPlate.spell2.enabled = true;
			loadoutPlate.spell3.enabled = true;
			loadoutPlate.spell1:GetHighlightTexture():SetAlpha(1);
			loadoutPlate.spell2:GetHighlightTexture():SetAlpha(1);
			loadoutPlate.spell3:GetHighlightTexture():SetAlpha(1);
			loadoutPlate.spell1:GetPushedTexture():SetAlpha(1);
			loadoutPlate.spell2:GetPushedTexture():SetAlpha(1);
			loadoutPlate.spell3:GetPushedTexture():SetAlpha(1);
			loadoutPlate.spell1.FlyoutArrow:Show();
			loadoutPlate.spell2.FlyoutArrow:Show();
			loadoutPlate.spell3.FlyoutArrow:Show();

			loadoutPlate.helpFrame:Hide();
			loadoutPlate.requirement:Hide();
			loadoutPlate.emptyslot:Hide();
			loadoutPlate.dragButton:Show();
		end
	end

	AnvilPetJournal.Loadout.Pet1.setButton:Hide();
	AnvilPetJournal.Loadout.Pet2.setButton:Hide();
	AnvilPetJournal.Loadout.Pet3.setButton:Hide();
end

function AnvilPetJournal_UpdateAll()
	AnvilPetJournal_UpdatePetList();
	AnvilPetJournal_UpdatePetLoadOut();
	AnvilPetJournal_UpdatePetCard(AnvilPetJournalPetCard);
	AnvilPetJournal_HidePetDropdown();
end

function AnvilPetJournal_UpdatePetList()
	local scrollFrame = AnvilPetJournal.listScroll;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local petButtons = scrollFrame.buttons;
	local pet, index;

	local numPets, numOwned = C_PetJournal.GetNumPets();
	AnvilPetJournal.AnvilPetCount.Count:SetText(numOwned);

	local summonedPetID = C_PetJournal.GetSummonedPetGUID();

	for i = 1,#petButtons do
		pet = petButtons[i];
		index = offset + i;
		if index <= numPets then
			local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, _, _, _, _, canBattle = C_PetJournal.GetPetInfoByIndex(index);
			local needsFanfare = petID and C_PetJournal.PetNeedsFanfare(petID);

			if customName then
				pet.name:SetText(customName);
				pet.name:SetHeight(12);
				pet.subName:Show();
				pet.subName:SetText(name);
			else
				pet.name:SetText(name);
				pet.name:SetHeight(30);
				pet.subName:Hide();
			end

			pet.icon:SetTexture(needsFanfare and COLLECTIONS_FANFARE_ICON or icon);
			pet.new:SetShown(needsFanfare);
			pet.newGlow:SetShown(needsFanfare);
			pet.petTypeIcon:SetTexture(GetPetTypeTexture(petType));

			if (favorite) then
				pet.dragButton.favorite:Show();
			else
				pet.dragButton.favorite:Hide();
			end

			if isOwned then
				local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID);

				pet.dragButton.levelBG:SetShown(canBattle);
				pet.dragButton.level:SetShown(canBattle);
				pet.dragButton.level:SetText(level);

				pet.icon:SetDesaturated(false);
				pet.name:SetFontObject("GameFontNormal");
				pet.petTypeIcon:SetShown(canBattle);
				pet.petTypeIcon:SetDesaturated(false);
				pet.dragButton:Enable();
				pet.iconBorder:Show();
				pet.iconBorder:SetVertexColor(ITEM_QUALITY_COLORS[rarity-1].r, ITEM_QUALITY_COLORS[rarity-1].g, ITEM_QUALITY_COLORS[rarity-1].b);
				if (health and health <= 0) then
					pet.isDead:Show();
				else
					pet.isDead:Hide();
				end
				if(isRevoked) then
					pet.dragButton.levelBG:Hide();
					pet.dragButton.level:Hide();
					pet.iconBorder:Hide();
					pet.icon:SetDesaturated(true);
					pet.petTypeIcon:SetDesaturated(true);
					pet.dragButton:Disable();
				end
			else
				pet.dragButton.levelBG:Hide();
				pet.dragButton.level:Hide();
				pet.icon:SetDesaturated(true);
				pet.iconBorder:Hide();
				pet.name:SetFontObject("GameFontDisable");
				pet.petTypeIcon:SetShown(canBattle);
				pet.petTypeIcon:SetDesaturated(true);
				pet.dragButton:Disable();
				pet.isDead:Hide();
			end

			if ( petID and petID == summonedPetID ) then
				pet.dragButton.ActiveTexture:Show();
			else
				pet.dragButton.ActiveTexture:Hide();
			end

			pet.petID = petID;
			pet.speciesID = speciesID;
			pet.index = index;
			pet.owned = isOwned;
			pet:Show();

			if AnvilPetJournalPetCard.petIndex == index then
				pet.selected = true;
				pet.selectedTexture:Show();
			else
				pet.selected = false;
				pet.selectedTexture:Hide()
			end

			if ( petID ) then
				local start, duration, enable = C_PetJournal.GetPetCooldownByGUID(pet.petID);
				if (start) then
					CooldownFrame_Set(pet.dragButton.Cooldown, start, duration, enable);
				end
			end
		else
			pet:Hide();
		end
	end

	local totalHeight = 46*(Anvil.db.global.collectionsWidth*ceil(numPets/Anvil.db.global.collectionsWidth)-11*(Anvil.db.global.collectionsWidth-1))+4
	HybridScrollFrame_Update(scrollFrame, totalHeight, scrollFrame:GetHeight());
end

function AnvilPetJournalListItem_OnClick(self, button)
	if ( IsModifiedClick("CHATLINK") ) then
		local id = self.petID;
		if (id) then
			local petLink = C_PetJournal.GetBattlePetLink(id);
			ChatEdit_InsertLink(petLink);
		end
	elseif button == "RightButton" then
		if self.owned then
			AnvilPetJournal_ShowPetDropdown(self.index, self, 80, 20);
		end
	else
		local type, petID = GetCursorInfo();
		if type == "battlepet" then
			AnvilPetJournal_UpdatePetLoadOut();
			ClearCursor();
		else
			AnvilPetJournal_ShowPetCard(self.index);
		end
	end
end

function AnvilPetJournalDragButton_OnClick(self, button)
	if ( IsModifiedClick("CHATLINK") ) then
		local id = self:GetParent().petID;
		if (id) then
			local petLink = C_PetJournal.GetBattlePetLink(id);
			ChatEdit_InsertLink(petLink);
		end
	elseif ( button == "RightButton" ) then
		local parent = self:GetParent();
		if ( parent.owned ) then
			AnvilPetJournal_ShowPetDropdown(parent.index, self, 0, 0);
		end
	else
		local type, petID = GetCursorInfo();
		if type == "battlepet" then
			AnvilPetJournal_UpdatePetLoadOut();
			ClearCursor();
		else
			AnvilPetJournalDragButton_OnDragStart(self);
		end
	end
end

function AnvilPetJournalPetLoadoutDragButton_OnClick(self, button)
	local loadout = self:GetParent();
	if (button == "RightButton" and loadout.petID) then
		AnvilPetJournal_ShowPetDropdown(nil, self, 0, 0, loadout.petID);
		return;
	end
	if ( IsModifiedClick("CHATLINK") ) then
		local id = self:GetParent().petID;
		if (id) then
			local petLink = C_PetJournal.GetBattlePetLink(id);
			ChatEdit_InsertLink(petLink);
		end
	else
		AnvilPetJournalDragButton_OnDragStart(self);
	end
end

function AnvilPetJournalDragButton_OnDragStart(self)
	if (not self:GetParent().petID) then
		return;
	end

	AnvilPetJournal_HidePetDropdown();
	C_PetJournal.PickupPet(self:GetParent().petID);

	for i=1,MAX_ACTIVE_PETS do
		local loadoutPlate = AnvilPetJournal.Loadout["Pet"..i];
		local petID, ability1ID, ability2ID, ability3ID, locked = C_PetJournal.GetPetLoadOutInfo(i);
		if(locked) then
			AnvilPetJournal.Loadout["Pet"..i].setButton:Hide();
		else
			AnvilPetJournal.Loadout["Pet"..i].setButton:Show();
		end
	end
end

function AnvilPetJournal_ShowPetDropdown(index, anchorTo, offsetX, offsetY, petID)
	if (index) then
		AnvilPetJournal.menuPetID = C_PetJournal.GetPetInfoByIndex(index);
	elseif (petID) then
		AnvilPetJournal.menuPetID = petID;
	else
		return;
	end
	ToggleDropDownMenu(1, nil, AnvilPetJournal.petOptionsMenu, anchorTo, offsetX, offsetY);
	PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
end

function AnvilPetJournal_HidePetDropdown()
	if (UIDropDownMenu_GetCurrentDropDown() == AnvilPetJournal.petOptionsMenu) then
		HideDropDownMenu(1);
	end
end

function AnvilPetJournal_ShowPetCardByID(petID)
	if (petID == nil) then
		AnvilPetJournal_ShowPetCard(1);
		return;
	end
	AnvilPetJournal_HidePetDropdown();
	AnvilPetJournalPetCard.petID = petID;
	AnvilPetJournalPetCard.speciesID = C_PetJournal.GetPetInfoByPetID(petID);

	AnvilPetJournal_FindPetCardIndex();
	AnvilPetJournal_UpdatePetCard(AnvilPetJournalPetCard);
	AnvilPetJournal_UpdatePetList();
	AnvilPetJournal_UpdateSummonButtonState();
end

function AnvilPetJournal_ShowPetCardBySpeciesID(speciesID)
	if (not speciesID) then
		AnvilPetJournal_ShowPetCard(1);
		return;
	end

	AnvilPetJournal_HidePetDropdown();
	AnvilPetJournalPetCard.petID = nil;
	AnvilPetJournalPetCard.speciesID = speciesID;
	AnvilPetJournal_FindPetCardIndex();
	AnvilPetJournal_UpdatePetCard(AnvilPetJournalPetCard);
	AnvilPetJournal_UpdatePetList();
	AnvilPetJournal_UpdateSummonButtonState();
end

function AnvilPetJournal_ShowPetCard(index)
	AnvilPetJournal_HidePetDropdown();
	AnvilPetJournalPetCard.petIndex = index;
	local owned;
	AnvilPetJournalPetCard.petID, AnvilPetJournalPetCard.speciesID, owned = C_PetJournal.GetPetInfoByIndex(index);
	if ( not owned ) then
		AnvilPetJournalPetCard.petID = nil;
	end
	AnvilPetJournal_UpdatePetCard(AnvilPetJournalPetCard);
	AnvilPetJournal_UpdatePetList();
	AnvilPetJournal_UpdateSummonButtonState();
end

function AnvilPetJournal_FindPetCardIndex()
	AnvilPetJournalPetCard.petIndex = nil;
	local numPets = C_PetJournal.GetNumPets();
	for i = 1,numPets do
		local petID, speciesID, owned = C_PetJournal.GetPetInfoByIndex(i);
		if (owned and petID == AnvilPetJournalPetCard.petID) or
			(not owned and speciesID == AnvilPetJournalPetCard.speciesID)  then
			AnvilPetJournalPetCard.petIndex = i;
			break;
		end
	end
end

function AnvilPetJournalPetCard_OnClick(self, button)
	local type, petID = GetCursorInfo();
	if type == "battlepet" then
		ClearCursor();
		return;
	end

	if ( IsModifiedClick("CHATLINK") ) then
		local id = AnvilPetJournalPetCard.petID;
		if (id) then
			local petLink = C_PetJournal.GetBattlePetLink(id);
			ChatEdit_InsertLink(petLink);
		end
	elseif button == "RightButton" then
		if ( AnvilPetJournalPetCard.petID ) then
			AnvilPetJournal_ShowPetDropdown(AnvilPetJournalPetCard.petIndex, self, 0, 0);
		end
	else
		AnvilPetJournalDragButton_OnDragStart(self);
	end
end

function AnvilPetJournal_UpdatePetCard(self, forceSceneChange)
	AnvilPetJournal.SpellSelect:Hide();

	if (not AnvilPetJournalPetCard.petID and not AnvilPetJournalPetCard.speciesID) then
		self.PetInfo.name:SetText(PET_JOURNAL_CARD_NAME_DEFAULT);
		self.PetInfo.subName:SetText("");
		self.PetInfo.subName:Hide();
		self.PetInfo.level:Hide();
		self.PetInfo.levelBG:Hide();
		self.PetInfo.qualityBorder:Hide();
		self.PetInfo.favorite:Hide();
		self.PetInfo.icon:Hide();

		self.TypeInfo:Hide();

		self.modelScene:Hide();

		self.AbilitiesBG1:Hide();
		self.AbilitiesBG2:Hide();
		self.AbilitiesBG3:Hide();
		self.CannotBattleText:Hide();
		for i=1,NUM_PET_ABILITIES do
			self["spell"..i]:Hide();
		end

		self.HealthFrame:Hide();
		self.PowerFrame:Hide();
		self.SpeedFrame:Hide();
		self.QualityFrame:Hide();

		self.xpBar:Hide();
		return;
	end

	self.PetInfo.icon:Show();

	local isDead = false;
	local needsFanfare = false;
	local speciesID, customName, level, name, icon, petType, creatureID, xp, maxXp, displayID, isFavorite, sourceText, description, isWild, canBattle, tradable, unique;
	if AnvilPetJournalPetCard.petID then
		speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique = C_PetJournal.GetPetInfoByPetID(AnvilPetJournalPetCard.petID);
		if ( not speciesID ) then
			return;
		end
		self.PetInfo.level:SetShown(canBattle);
		self.PetInfo.levelBG:SetShown(canBattle);
		self.PetInfo.level:SetText(level);

		needsFanfare = C_PetJournal.PetNeedsFanfare(AnvilPetJournalPetCard.petID);

		self.xpBar:SetShown(level < MAX_PET_LEVEL and canBattle);
		if (level < MAX_PET_LEVEL) then
			self.xpBar:SetMinMaxValues(0, maxXp);
			self.xpBar:SetValue(xp);
			local display = GetCVar("statusTextDisplay")
			if (display == "BOTH") then
				self.xpBar.rankText:SetFormattedText(PET_BATTLE_CURRENT_XP_FORMAT_BOTH, xp, maxXp, xp/maxXp*100);
			elseif(display == "PERCENT") then
				self.xpBar.rankText:SetFormattedText(PET_BATTLE_CURRENT_XP_FORMAT_PERCENT, xp/maxXp*100);
			else
				self.xpBar.rankText:SetFormattedText(PET_BATTLE_CURRENT_XP_FORMAT_VERBOSE, xp, maxXp);
			end
			self.xpBar.tooltip = format(PET_BATTLE_CURRENT_XP_FORMAT_TOOLTIP, xp, maxXp, xp/maxXp*100);
		end

		local health, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(AnvilPetJournalPetCard.petID);
		self.HealthFrame:SetShown(canBattle);
		self.PowerFrame:SetShown(canBattle);
		self.SpeedFrame:SetShown(canBattle);

		self.HealthFrame.healthBar:SetMinMaxValues(0, maxHealth);
		self.HealthFrame.healthBar:SetValue(health);
		self.HealthFrame.health:SetText(maxHealth);

		isDead = health <= 0;

		self.PowerFrame.power:SetText(power);
		self.SpeedFrame.speed:SetText(speed);
		if ( canBattle ) then
			self.QualityFrame.quality:SetText(_G["BATTLE_PET_BREED_QUALITY"..rarity]);
			local color = ITEM_QUALITY_COLORS[rarity-1];
			self.QualityFrame.quality:SetVertexColor(color.r, color.g, color.b);
			self.QualityFrame:Show();
			self.PetInfo.qualityBorder:Show();
			self.PetInfo.qualityBorder:SetVertexColor(color.r, color.g, color.b);
		else
			self.QualityFrame:Hide();
			self.PetInfo.qualityBorder:Hide();
		end
	else
		speciesID = AnvilPetJournalPetCard.speciesID;
		name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique, _, displayID = C_PetJournal.GetPetInfoBySpeciesID(AnvilPetJournalPetCard.speciesID);
		level = 1;
		self.PetInfo.level:Hide();
		self.PetInfo.levelBG:Hide();

		self.xpBar:Hide();
		self.HealthFrame:Hide();
		self.PowerFrame:Hide();
		self.SpeedFrame:Hide();
		self.QualityFrame:Hide();
		self.PetInfo.qualityBorder:Hide();
	end
	self.PetInfo.isDead:SetShown(isDead);

	self.TypeInfo:SetShown(canBattle);

	self.TypeInfo.type:SetText(_G["BATTLE_PET_NAME_"..petType]);
	self.TypeInfo.typeIcon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType]);
	self.TypeInfo.abilityID = PET_BATTLE_PET_TYPE_PASSIVES[petType];
	self.TypeInfo.petID = AnvilPetJournalPetCard.petID;
	self.TypeInfo.speciesID = speciesID;

	if (isFavorite) then
		self.PetInfo.favorite:Show();
	else
		self.PetInfo.favorite:Hide();
	end

	if customName then
		self.PetInfo.name:SetText(customName);
		self.PetInfo.name:SetHeight(24);
		self.PetInfo.subName:Show();
		self.PetInfo.subName:SetText(name);
	else
		self.PetInfo.name:SetText(name);
		self.PetInfo.name:SetHeight(32);
		self.PetInfo.subName:Hide();
	end


	self.PetInfo.new:SetShown(needsFanfare);
	self.PetInfo.newGlow:SetShown(needsFanfare);

	if needsFanfare then
		self.PetInfo.icon:SetTexture(COLLECTIONS_FANFARE_ICON);
		local offsetX = math.min(self.PetInfo.name:GetStringWidth(), self.PetInfo.name:GetWidth());
		self.PetInfo.new:SetPoint("BOTTOMLEFT", self.PetInfo.name, "BOTTOMLEFT", offsetX + 8, 0);
	else
		self.PetInfo.icon:SetTexture(icon);
	end

	self.PetInfo.sourceText = sourceText;
	self.PetInfo.tradable = tradable;
	self.PetInfo.unique = unique;

	if ( description ~= "" ) then
		self.PetInfo.description = format([["%s"]], description);
	else
		self.PetInfo.description = nil;
	end
	self.PetInfo.speciesName = name;

	self.modelScene:Show();
	local modelChanged = false;
	if ( displayID ~= self.displayID or forceSceneChange ) then
		self.displayID = displayID;

		local cardModelSceneID, loadoutModelSceneID = C_PetJournal.GetPetModelSceneInfoBySpeciesID(speciesID);

		self.modelScene:TransitionToModelSceneID(cardModelSceneID, CAMERA_TRANSITION_TYPE_IMMEDIATE, CAMERA_MODIFICATION_TYPE_MAINTAIN, forceSceneChange);

		local battlePetActor = self.modelScene:GetActorByTag("unwrapped");
		if ( battlePetActor ) then
			battlePetActor:SetModelByCreatureDisplayID(displayID);
			battlePetActor:SetAnimationBlendOperation(LE_MODEL_BLEND_OPERATION_NONE);
		end

		modelChanged = true;
	end

	if ( modelChanged or self.modelScene.wasDead ~= isDead ) then
		local battlePetActor = self.modelScene:GetActorByTag("unwrapped");
		if ( battlePetActor ) then
			if ( isDead ) then
				battlePetActor:SetAnimation(6, -1);
			else
				battlePetActor:SetAnimation(0, -1);
			end
		end

		self.modelScene.wasDead = isDead;
	end
	self.modelScene:PrepareForFanfare(needsFanfare);

	self.AbilitiesBG1:SetShown(canBattle);
	self.AbilitiesBG2:SetShown(canBattle);
	self.AbilitiesBG3:SetShown(canBattle);
	self.CannotBattleText:SetShown(not canBattle);

	local abilities, levels = C_PetJournal.GetPetAbilityList(speciesID);
	for i=1,NUM_PET_ABILITIES do
		local spellFrame = self["spell"..i];
		if abilities[i] and canBattle then
			local name, icon, petType = C_PetJournal.GetPetAbilityInfo(abilities[i]);
			local isNotUsable = not level or level < levels[i];
			spellFrame.icon:SetTexture(icon);
			spellFrame.icon:SetDesaturated(isNotUsable);
			spellFrame.LevelRequirement:SetText(levels[i]);
			spellFrame.LevelRequirement:SetShown(isNotUsable);
			spellFrame.BlackCover:SetShown(isNotUsable);
			if (not level or level < levels[i]) then
				spellFrame.additionalText = format(PET_ABILITY_REQUIRES_LEVEL, levels[i]);
			else
				spellFrame.additionalText = nil;
			end
			spellFrame.abilityID = abilities[i];
			spellFrame.petID = AnvilPetJournalPetCard.petID;
			spellFrame.speciesID = speciesID;
			spellFrame:Show();
		else
			spellFrame:Hide();
		end
	end
end

function AnvilPetJournal_UnwrapPet(petID)
	if AnvilPetJournalPetCard.modelScene:IsUnwrapAnimating() or not C_PetJournal.PetNeedsFanfare(petID) then
		return;
	end

	AnvilPetJournal_ShowPetCardByID(petID);

	local function OnFinishedCallback()
		C_PetJournal.ClearFanfare(petID);
		AnvilPetJournal_ShowPetCardByID(petID);
	end

	AnvilPetJournalPetCard.modelScene:StartUnwrapAnimation(OnFinishedCallback);
end

function AnvilPetJournal_SetPendingCage(petID)
	AnvilPetJournal.pendingCage = petID;
end

function AnvilPetJournal_ClearPendingCage()
	AnvilPetJournal.pendingCage = nil;
end

function AnvilPetJournal_IsPendingCage(petID)
	return AnvilPetJournal.pendingCage and AnvilPetJournal.pendingCage == petID;
end

function AnvilPetJournalFilterDropDown_OnLoad(self)
	UIDropDownMenu_Initialize(self, AnvilPetJournalFilterDropDown_Initialize, "MENU");
end


function AnvilPetJournalFilterDropDown_Initialize(self, level)
	local info = UIDropDownMenu_CreateInfo();
	info.keepShownOnClick = true;

	if level == 1 then

		info.text = COLLECTED
		info.func = 	function(_, _, _, value)
							C_PetJournal.SetFilterChecked(LE_PET_JOURNAL_FILTER_COLLECTED, value);
						end
		info.checked = C_PetJournal.IsFilterChecked(LE_PET_JOURNAL_FILTER_COLLECTED);
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level);

		info.disabled = nil;

		info.text = NOT_COLLECTED;
		info.func = 	function(_, _, _, value)
							C_PetJournal.SetFilterChecked(LE_PET_JOURNAL_FILTER_NOT_COLLECTED, value);
						end
		info.checked = C_PetJournal.IsFilterChecked(LE_PET_JOURNAL_FILTER_NOT_COLLECTED);
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level);

		info.checked = 	nil;
		info.isNotRadio = nil;
		info.func =  nil;
		info.hasArrow = true;
		info.notCheckable = true;

		info.text = PET_FAMILIES;
		info.value = 1;
		UIDropDownMenu_AddButton(info, level);

		info.text = SOURCES;
		info.value = 2;
		UIDropDownMenu_AddButton(info, level);

		info.text = RAID_FRAME_SORT_LABEL;
		info.value = 3;
		UIDropDownMenu_AddButton(info, level);

	else
		if UIDROPDOWNMENU_MENU_VALUE == 1 then
			info.hasArrow = false;
			info.isNotRadio = true;
			info.notCheckable = true;

			info.text = CHECK_ALL;
			info.func = function()
							C_PetJournal.SetAllPetTypesChecked(true);
							UIDropDownMenu_Refresh(AnvilPetJournalFilterDropDown, 1, 2);
						end
			UIDropDownMenu_AddButton(info, level);

			info.text = UNCHECK_ALL;
			info.func = function()
							C_PetJournal.SetAllPetTypesChecked(false);
							UIDropDownMenu_Refresh(AnvilPetJournalFilterDropDown, 1, 2);
						end
			UIDropDownMenu_AddButton(info, level);

			info.notCheckable = false;
			local numTypes = C_PetJournal.GetNumPetTypes();
			for i=1,numTypes do
				info.text = _G["BATTLE_PET_NAME_"..i];
				info.func = function(_, _, _, value)
							C_PetJournal.SetPetTypeFilter(i, value);
						end
				info.checked = function() return C_PetJournal.IsPetTypeChecked(i) end;
				UIDropDownMenu_AddButton(info, level);
			end
		elseif UIDROPDOWNMENU_MENU_VALUE == 2 then
			info.hasArrow = false;
			info.isNotRadio = true;
			info.notCheckable = true;

			info.text = CHECK_ALL;
			info.func = function()
							C_PetJournal.SetAllPetSourcesChecked(true);
							UIDropDownMenu_Refresh(AnvilPetJournalFilterDropDown, 2, 2);
						end
			UIDropDownMenu_AddButton(info, level);

			info.text = UNCHECK_ALL;
			info.func = function()
							C_PetJournal.SetAllPetSourcesChecked(false);
							UIDropDownMenu_Refresh(AnvilPetJournalFilterDropDown, 2, 2);
						end
			UIDropDownMenu_AddButton(info, level);

			info.notCheckable = false;
			local numSources = C_PetJournal.GetNumPetSources();
			for i=1,numSources do
				info.text = _G["BATTLE_PET_SOURCE_"..i];
				info.func = function(_, _, _, value)
							C_PetJournal.SetPetSourceChecked(i, value);
						end
				info.checked = function() return C_PetJournal.IsPetSourceChecked(i) end;
				UIDropDownMenu_AddButton(info, level);
			end
		elseif UIDROPDOWNMENU_MENU_VALUE == 3 then
			info.hasArrow = false;
			info.isNotRadio = nil;
			info.notCheckable = nil;
			info.keepShownOnClick = nil;

			info.text = NAME;
			info.func = function()
							C_PetJournal.SetPetSortParameter(LE_SORT_BY_NAME);
							AnvilPetJournal_UpdatePetList();
						end
			info.checked = function() return C_PetJournal.GetPetSortParameter() == LE_SORT_BY_NAME end;
			UIDropDownMenu_AddButton(info, level);

			info.text = LEVEL;
			info.func = function()
							C_PetJournal.SetPetSortParameter(LE_SORT_BY_LEVEL);
							AnvilPetJournal_UpdatePetList();
						end
			info.checked = function() return C_PetJournal.GetPetSortParameter() == LE_SORT_BY_LEVEL end;
			UIDropDownMenu_AddButton(info, level);

			info.text = RARITY;
			info.func = function()
							C_PetJournal.SetPetSortParameter(LE_SORT_BY_RARITY);
							AnvilPetJournal_UpdatePetList();
						end
			info.checked = function() return C_PetJournal.GetPetSortParameter() == LE_SORT_BY_RARITY end;
			UIDropDownMenu_AddButton(info, level);

			info.text = TYPE;
			info.func = function()
							C_PetJournal.SetPetSortParameter(LE_SORT_BY_PETTYPE);
							AnvilPetJournal_UpdatePetList();
						end
			info.checked = function() return C_PetJournal.GetPetSortParameter() == LE_SORT_BY_PETTYPE end;
			UIDropDownMenu_AddButton(info, level);
		end
	end
end

function AnvilPetOptionsMenu_Init(self, level)
	local info = UIDropDownMenu_CreateInfo();
	info.notCheckable = true;

	local needsFanfare = AnvilPetJournal.menuPetID and C_PetJournal.PetNeedsFanfare(AnvilPetJournal.menuPetID);

	if needsFanfare then
		info.text = UNWRAP;
		info.func = function() AnvilPetJournal_UnwrapPet(AnvilPetJournal.menuPetID); end
		UIDropDownMenu_AddButton(info, level);
	else
		local isRevoked = AnvilPetJournal.menuPetID and C_PetJournal.PetIsRevoked(AnvilPetJournal.menuPetID);
		local isLockedForConvert = AnvilPetJournal.menuPetID and C_PetJournal.PetIsLockedForConvert(AnvilPetJournal.menuPetID);

		if (not isRevoked and not isLockedForConvert) then
			info.text = BATTLE_PET_SUMMON;
			if (AnvilPetJournal.menuPetID and C_PetJournal.GetSummonedPetGUID() == AnvilPetJournal.menuPetID) then
				info.text = PET_DISMISS;
			end
			info.func = function() C_PetJournal.SummonPetByGUID(AnvilPetJournal.menuPetID); end
			if (AnvilPetJournal.menuPetID and not C_PetJournal.PetIsSummonable(AnvilPetJournal.menuPetID)) then
				info.disabled = true;
			end
			UIDropDownMenu_AddButton(info, level);
			info.disabled = nil;
		end

		if (not isRevoked and not isLockedForConvert) then
			info.text = BATTLE_PET_RENAME
			info.func = 	function() StaticPopup_Show("BATTLE_PET_RENAME", nil, nil, AnvilPetJournal.menuPetID); end
			info.disabled = not C_PetJournal.IsJournalUnlocked();
			UIDropDownMenu_AddButton(info, level);
			info.disabled = nil;
		end

		local isFavorite = AnvilPetJournal.menuPetID and C_PetJournal.PetIsFavorite(AnvilPetJournal.menuPetID);
		if (isFavorite or (not isRevoked and not isLockedForConvert)) then
			if (isFavorite) then
				info.text = BATTLE_PET_UNFAVORITE;
				info.func = function()
					C_PetJournal.SetFavorite(AnvilPetJournal.menuPetID, 0);
				end
			else
				info.text = BATTLE_PET_FAVORITE;
				info.func = function()
					C_PetJournal.SetFavorite(AnvilPetJournal.menuPetID, 1);
				end
			end
			info.disabled = not C_PetJournal.IsJournalUnlocked();
			UIDropDownMenu_AddButton(info, level);
			info.disabled = nil;
		end

		if(AnvilPetJournal.menuPetID and C_PetJournal.PetCanBeReleased(AnvilPetJournal.menuPetID)) then
			info.text = BATTLE_PET_RELEASE;
			info.func = function() StaticPopup_Show("BATTLE_PET_RELEASE", PetJournalUtil_GetDisplayName(AnvilPetJournal.menuPetID), nil, AnvilPetJournal.menuPetID); end
			if (C_PetJournal.PetIsSlotted(AnvilPetJournal.menuPetID) or C_PetBattles.IsInBattle() or not C_PetJournal.IsJournalUnlocked()) then
				info.disabled = true;
			else
				info.disabled = nil;
			end
			UIDropDownMenu_AddButton(info, level);
			info.disabled = nil;
		end

		if(AnvilPetJournal.menuPetID and C_PetJournal.PetIsTradable(AnvilPetJournal.menuPetID)) then
			info.text = BATTLE_PET_PUT_IN_CAGE;
			info.func = function() StaticPopup_Show("BATTLE_PET_PUT_IN_CAGE", nil, nil, AnvilPetJournal.menuPetID); end

			info.disabled = nil;
			if (not info.disabled and C_PetJournal.PetIsSlotted(AnvilPetJournal.menuPetID)) then
				info.disabled = true;
				info.text = BATTLE_PET_PUT_IN_CAGE_SLOTTED;
			end
			if (not info.disabled and C_PetJournal.PetIsHurt(AnvilPetJournal.menuPetID)) then
				info.disabled = true;
				info.text = BATTLE_PET_PUT_IN_CAGE_HEALTH;
			end
			UIDropDownMenu_AddButton(info, level)
			info.disabled = nil;
		end
	end

	info.text = CANCEL
	info.func = nil
	UIDropDownMenu_AddButton(info, level)
end

local PET_JOURNAL_ABILITY_INFO = SharedPetBattleAbilityTooltip_GetInfoTable();

function PET_JOURNAL_ABILITY_INFO:GetAbilityID()
	return self.abilityID;
end

function PET_JOURNAL_ABILITY_INFO:IsInBattle()
	return false;
end

function PET_JOURNAL_ABILITY_INFO:GetHealth(target)
	self:EnsureTarget(target);
	if ( self.petID ) then
		local health, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(self.petID);
		return health;
	else
		return self:GetMaxHealth(target);
	end
end

function PET_JOURNAL_ABILITY_INFO:GetMaxHealth(target)
	self:EnsureTarget(target);
	if ( self.petID ) then
		local health, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(self.petID);
		return maxHealth;
	else
		return 100;
	end
end

function PET_JOURNAL_ABILITY_INFO:GetAttackStat(target)
	self:EnsureTarget(target);
	if ( self.petID ) then
		local health, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(self.petID);
		return power;
	else
		return 0;
	end
end

function PET_JOURNAL_ABILITY_INFO:GetSpeedStat(target)
	self:EnsureTarget(target);
	if ( self.petID ) then
		local health, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(self.petID);
		return speed;
	else
		return 0;
	end
end

function PET_JOURNAL_ABILITY_INFO:GetPetOwner(target)
	self:EnsureTarget(target);
	return LE_BATTLE_PET_ALLY;
end

function PET_JOURNAL_ABILITY_INFO:GetPetType(target)
	self:EnsureTarget(target);
	if ( not self.speciesID ) then
		GMError("No species id found");
		return 1;
	end
	local _, _, petType = C_PetJournal.GetPetInfoBySpeciesID(self.speciesID);
	return petType;
end

function PET_JOURNAL_ABILITY_INFO:EnsureTarget(target)
	if ( target == "default" ) then
		target = "self";
	elseif ( target == "affected" ) then
		target = "enemy";
	end
	if ( target ~= "self" ) then
		GMError("Only \"self\" unit supported out of combat");
	end
end

local journalAbilityInfo = {};
setmetatable(journalAbilityInfo, {__index = PET_JOURNAL_ABILITY_INFO});

function AnvilPetJournal_ShowAbilityTooltip(self, abilityID, speciesID, petID, additionalText)
	if ( abilityID and abilityID > 0 ) then
		journalAbilityInfo.abilityID = abilityID;
		journalAbilityInfo.speciesID = speciesID;
		journalAbilityInfo.petID = petID;
		AnvilPetJournalPrimaryAbilityTooltip:ClearAllPoints();
		AnvilPetJournalPrimaryAbilityTooltip:SetPoint("TOPLEFT", self, "TOPRIGHT", 5, 0);
		AnvilPetJournalPrimaryAbilityTooltip.anchoredTo = self;
		SharedPetBattleAbilityTooltip_SetAbility(AnvilPetJournalPrimaryAbilityTooltip, journalAbilityInfo, additionalText);
		AnvilPetJournalPrimaryAbilityTooltip:Show();
	end
end

function AnvilPetJournal_GetPetAbilityHyperlink(abilityID, petID)
	local maxHealth, power, speed, _;
	if ( petID ) then
		_, maxHealth, power, speed, _ = C_PetJournal.GetPetStats(petID);
	else
		maxHealth, power, speed = 100, 0, 0;
	end
	return GetBattlePetAbilityHyperlink(abilityID, maxHealth, power, speed);
end

local CompareInfo1 = {};
local CompareInfo2 = {};
setmetatable(CompareInfo1, {__index = PET_JOURNAL_ABILITY_INFO});
setmetatable(CompareInfo2, {__index = PET_JOURNAL_ABILITY_INFO});

function AnvilPetJournal_ShowAbilityCompareTooltip(abilityID1, abilityID2, speciesID, petID)
	if ( abilityID1 and abilityID2 ) then
		CompareInfo1.abilityID = abilityID1;
		CompareInfo1.speciesID = speciesID;
		CompareInfo1.petID = petID;

		CompareInfo2.abilityID = abilityID2;
		CompareInfo2.speciesID = speciesID;
		CompareInfo2.petID = petID;

		AnvilPetJournalSecondaryAbilityTooltip:ClearAllPoints();
		AnvilPetJournalSecondaryAbilityTooltip:SetPoint("TOPLEFT", AnvilPetJournal.SpellSelect, "RIGHT", -15, 0);
		AnvilPetJournalPrimaryAbilityTooltip:ClearAllPoints();
		AnvilPetJournalPrimaryAbilityTooltip:SetPoint("BOTTOM", AnvilPetJournalSecondaryAbilityTooltip, "TOP", 0, 5);

		AnvilPetJournalPrimaryAbilityTooltip.anchoredTo = AnvilPetJournal.SpellSelect;
		SharedPetBattleAbilityTooltip_SetAbility(AnvilPetJournalPrimaryAbilityTooltip, CompareInfo1);

		SharedPetBattleAbilityTooltip_SetAbility(AnvilPetJournalSecondaryAbilityTooltip, CompareInfo2);
		AnvilPetJournalPrimaryAbilityTooltip:Show();
		AnvilPetJournalSecondaryAbilityTooltip:Show();
	end
end

function AnvilPetJournal_HideAbilityTooltip(self)
	if ( AnvilPetJournalPrimaryAbilityTooltip.anchoredTo == self or not self ) then
		AnvilPetJournalPrimaryAbilityTooltip:Hide();
	end
end

function AnvilPetJournal_UpdateFindBattleButton()
	AnvilPetJournal.FindBattleButton:SetEnabled(C_PetJournal.IsFindBattleEnabled() and C_PetJournal.IsJournalUnlocked());
end

function AnvilPetJournalAchievementStatus_OnEnter(self)
	AnvilPetJournal.AchievementStatus.highlight:Show();
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetText(BATTLE_PETS_ACHIEVEMENT, HIGHLIGHT_FONT_COLOR:GetRGB());
	GameTooltip:AddLine(BATTLE_PETS_ACHIEVEMENT_TOOLTIP, nil, nil, nil, true);
	GameTooltip:Show();
end

function AnvilPetJournalSummonButton_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetText(self:GetText(), HIGHLIGHT_FONT_COLOR:GetRGB());

	local needsFanFare = AnvilPetJournalPetCard.petID and C_PetJournal.PetNeedsFanfare(AnvilPetJournalPetCard.petID);
	if needsFanFare then
		GameTooltip:AddLine(BATTLE_PETS_UNWRAP_TOOLTIP, nil, nil, nil, true);
	else
		GameTooltip:AddLine(BATTLE_PETS_SUMMON_TOOLTIP, nil, nil, nil, true);
	end
	GameTooltip:Show();
end

AnvilPetJournal_HelpPlate = {
	FramePos = { x = 0,          y = -22 },
	FrameSize = { width = 700, height = 580 },
	[1] = { ButtonPos = { x = 26,	y = -75 },  HighLightBox = { x = 10, y = -72, width = 247, height = 50 },	 ToolTipDir = "RIGHT",  ToolTipText = PET_JOURNAL_HELP_1 },
	[2] = { ButtonPos = { x = 105,  y = -300 }, HighLightBox = { x = 10, y = -125, width = 247, height = 430 },  ToolTipDir = "DOWN",   ToolTipText = PET_JOURNAL_HELP_2 },
	[3] = { ButtonPos = { x = 470,  y = -245 }, HighLightBox = { x = 290, y = -215, width = 400, height = 340 }, ToolTipDir = "DOWN",   ToolTipText = PET_JOURNAL_HELP_3 },
	[4] = { ButtonPos = { x = 525,  y = -546},  HighLightBox = { x = 550, y = -556, width = 150, height = 26 },  ToolTipDir = "UP",		ToolTipText = PET_JOURNAL_HELP_4 },
	[5] = { ButtonPos = { x = 470,  y = -95 },  HighLightBox = { x = 290, y = -45, width = 400, height = 160 },  ToolTipDir = "RIGHT",  ToolTipText = PET_JOURNAL_HELP_5 },
	[6] = { ButtonPos = { x = 525,  y = 0 },	HighLightBox = { x = 550, y = 0, width = 150, height = 40 },     ToolTipDir = "LEFT",   ToolTipText = PET_JOURNAL_HELP_6 },
}

function AnvilPetJournalFindBattle_Update(self)
	local queueState = C_PetBattles.GetPVPMatchmakingInfo();
	if ( queueState == "queued" or queueState == "proposal" or queueState == "suspended" ) then
		self:SetText(LEAVE_QUEUE);
	else
		self:SetText(FIND_BATTLE);
	end
end

function AnvilPetJournalFindBattle_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetText(FIND_BATTLE, 1, 1, 1);
	GameTooltip:AddLine(BATTLE_PETS_FIND_BATTLE_TOOLTIP, nil, nil, nil, true);

	if (not C_PetJournal.IsFindBattleEnabled()) then
		GameTooltip:AddLine(BATTLE_PET_FIND_BATTLE_DISABLED, RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, true);
	elseif (not C_PetJournal.IsJournalUnlocked()) then
		GameTooltip:AddLine(BATTLE_PET_FIND_BATTLE_READONLY, RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, true);
	end

	GameTooltip:Show();
end


function AnvilPetJournal_UpdatePetAbility(abilityFrame, abilityID, petID)
	local speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType = C_PetJournal.GetPetInfoByPetID(petID);

	local requiredLevel = AnvilPetJournalLoadout_GetRequiredLevel(abilityFrame:GetParent(), abilityID);

	local name, icon, typeEnum = C_PetJournal.GetPetAbilityInfo(abilityID);
	abilityFrame.icon:SetTexture(icon);
	abilityFrame.abilityID = abilityID;
	abilityFrame.petID = petID;
	abilityFrame.speciesID = speciesID;
	abilityFrame.selected:Hide();

	local levelTooLow = requiredLevel > level;
	abilityFrame.icon:SetDesaturated(levelTooLow);
	abilityFrame.BlackCover:SetShown(levelTooLow);
	abilityFrame.LevelRequirement:SetText(requiredLevel);
	abilityFrame.LevelRequirement:SetShown(levelTooLow);
	if ( levelTooLow ) then
		abilityFrame.additionalText = format(PET_ABILITY_REQUIRES_LEVEL, requiredLevel);
	else
		abilityFrame.additionalText = nil;
	end
end


function AnvilPetJournalLoadout_GetRequiredLevel(loadoutPlate, abilityID)
	for i=1, NUM_PET_ABILITIES do
		if ( loadoutPlate.abilities[i] == abilityID ) then
			return loadoutPlate.abilityLevels[i];
		end
	end
	return 0;
end

function AnvilPetJournalHealPetButton_OnLoad(self)
	self.spellID = HEAL_PET_SPELL;
	local spellName, spellSubname, spellIcon = GetSpellInfo(self.spellID);
	self.texture:SetTexture(spellIcon);
	self.spellname:SetText(spellName);
	self:SetAttribute("type", "spell")
	self:SetAttribute("spell", self.spellID)
	self:RegisterForClicks("LeftButtonUp", "RightButtonUp")
end

function AnvilPetJournalHealPetButton_OnShow(self)
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	self:RegisterEvent("SPELLS_CHANGED");
	self:RegisterEvent("PET_BATTLE_OPENING_START");
	self:RegisterEvent("PET_BATTLE_CLOSE");
	AnvilPetJournalHealPetButton_UpdateCooldown(self);
	AnvilPetJournalHealPetButton_UpdateUsability(self);
end

function AnvilPetJournalHealPetButton_OnHide(self)
	self:UnregisterEvent("PET_BATTLE_OPENING_START");
	self:UnregisterEvent("PET_BATTLE_CLOSE");
	self:UnregisterEvent("SPELL_UPDATE_COOLDOWN");
	if (self:IsEventRegistered("SPELLS_CHANGED")) then
		self:UnregisterEvent("SPELLS_CHANGED");
	end
end

function AnvilPetJournalHealPetButton_OnDragStart(self)
	PickupSpell(self.spellID);
end

function AnvilPetJournalHealPetButton_UpdateUsability(self)
	if (IsSpellKnown(self.spellID) and C_PetJournal.IsJournalUnlocked()) then
		if (C_PetBattles.IsInBattle() or not IsUsableSpell(self.spellID) ) then
			self:SetButtonState("NORMAL", true);
			self.texture:SetDesaturated(true);
			self.LockIcon:Show();
			self.BlackCover:Show();
		else
			self:SetButtonState("NORMAL", false);
			self.texture:SetDesaturated(false);
			self.LockIcon:Hide();
			self.BlackCover:Hide();
			self:RegisterForClicks("LeftButtonUp", "RightButtonUp");
			self:RegisterForDrag("LeftButton");
		end
		if (self:IsEventRegistered("SPELLS_CHANGED")) then
			self:UnregisterEvent("SPELLS_CHANGED");
		end
	else
		self.BlackCover:Show();
		self.texture:SetDesaturated(true);
	end
end

function AnvilPetJournalHealPetButton_OnEvent(self, event, ...)
	if ( event == "SPELL_UPDATE_COOLDOWN" ) then
		AnvilPetJournalHealPetButton_UpdateCooldown(self);
		if ( GameTooltip:GetOwner() == self ) then
			AnvilPetJournalHealPetButton_OnEnter(self);
		end
	elseif ( event == "SPELLS_CHANGED" or event == "PET_BATTLE_OPENING_START" or event == "PET_BATTLE_CLOSE" ) then
		AnvilPetJournalHealPetButton_UpdateUsability(self);
	end
end

function AnvilPetJournalHealPetButton_UpdateCooldown(self)
	local cooldown = self.cooldown;
	local start, duration, enable = GetSpellCooldown(self.spellID);
	CooldownFrame_Set(cooldown, start, duration, enable);
end

function AnvilPetJournalHealPetButton_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	GameTooltip:SetSpellByID(self.spellID);
	if (not IsSpellKnown(self.spellID)) then
		GameTooltip:AddLine(PET_BATTLE_HEAL_SPELL_UNKNOWN, RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, true);
		GameTooltip:Show();
	elseif (not C_PetJournal.IsJournalUnlocked()) then
		GameTooltip:AddLine(PET_JOURNAL_HEAL_SPELL_LOCKED, RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, true);
		GameTooltip:Show();
	elseif (C_PetBattles.IsInBattle()) then
		GameTooltip:AddLine(PET_JOURNAL_HEAL_SPELL_IN_BATTLE, RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, true);
		GameTooltip:Show();
	end
	self.UpdateTooltip = AnvilPetJournalHealPetButton_OnEnter;
end
