AnvilSpellbook = Anvil:NewModule("AnvilSpellbook", "AceConsole-3.0", "AceEvent-3.0")
local _G = _G
local offset, numSpells, enabledCheck, spellbookSize, postCombatLaunch
local AnvilSpellbookInfo = {}
local ANVILSPELLBOOK_PAGENUMBERS = {}
AnvilSpellbookInfo["spell"] 	= { 	showFrames = {"AnvilSpellbookSpellIconsFrame", "AnvilSpellbookSideTabsFrame", "AnvilSpellbookPageNavigationFrame"},
								title = SPELLBOOK,
								updateFunc = function() AnvilSpellbook_UpdatePlayerTab(); end};
AnvilSpellbookInfo["professions"] = { 	showFrames = {"AnvilSpellbookProfessionFrame"},
								title = TRADE_SKILLS,
								updateFunc = function() AnvilSpellbook_UpdateProfTab(); end};
AnvilSpellbookInfo["pet"] 	= { 	showFrames = {"AnvilSpellbookSpellIconsFrame", "AnvilSpellbookPageNavigationFrame"},
								title = PET,
								updateFunc =  function() AnvilSpellbook_UpdatePetTab(); end};
local AnvilSpellbookFrames = {	"AnvilSpellbookSpellIconsFrame", "AnvilSpellbookProfessionFrame",  "AnvilSpellbookSideTabsFrame", "AnvilSpellbookPageNavigationFrame" };

function AnvilSpellbook:OnEnable()
	if not Anvil.db.global.spellbook then
		self:OnDisable()
	else
		spellbookSize = Anvil.db.global.spellbookWidth*Anvil.db.global.spellbookHeight
		AnvilSpellbookFrame = CreateFrame("Frame", "AnvilSpellbookFrame", UIParent, "AnvilSpellbookFrameTemplate")
		AnvilSpellbookFrame:SetFrameStrata("HIGH")
		AnvilSpellbookFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 16, -118)
		tinsert(UISpecialFrames, "AnvilSpellbookFrame")
		AnvilSpellbookFrame:RegisterEvent("SPELLS_CHANGED")
		AnvilSpellbookFrame:RegisterEvent("LEARNED_SPELL_IN_TAB")
		AnvilSpellbookFrame:RegisterEvent("SKILL_LINES_CHANGED")
		AnvilSpellbookFrame:RegisterEvent("PLAYER_GUILD_UPDATE")
		AnvilSpellbookFrame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
		AnvilSpellbookFrame:RegisterEvent("USE_GLYPH")
		AnvilSpellbookFrame:RegisterEvent("CANCEL_GLYPH_CAST")
		AnvilSpellbookFrame:RegisterEvent("ACTIVATE_GLYPH")
		AnvilSpellbookFrame:RegisterEvent("CURRENT_SPELL_CAST_CHANGED")
		AnvilSpellbookFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
		AnvilSpellbookFrame:RegisterEvent("PLAYER_REGEN_ENABLED")

		AnvilSpellbookFrame.bookType = "spell";
		ANVILSPELLBOOK_PAGENUMBERS[1] = 1;
		ANVILSPELLBOOK_PAGENUMBERS[2] = 1;
		ANVILSPELLBOOK_PAGENUMBERS[3] = 1;
		ANVILSPELLBOOK_PAGENUMBERS[4] = 1;
		ANVILSPELLBOOK_PAGENUMBERS[5] = 1;
		ANVILSPELLBOOK_PAGENUMBERS[6] = 1;
		ANVILSPELLBOOK_PAGENUMBERS[7] = 1;
		ANVILSPELLBOOK_PAGENUMBERS[8] = 1;
		ANVILSPELLBOOK_PAGENUMBERS["pet"] = 1;

		AnvilSpellbookFrame.selectedSkillLine = 2;
		AnvilSpellbookFrame.flashTabs = nil;

		SetPortraitToTexture(AnvilSpellbookFramePortrait, "Interface\\Spellbook\\Spellbook-Icon");

		hooksecurefunc("ToggleSpellBook", AnvilSpellbook_ToggleSpellbook)
		enabledCheck = true
		if not InCombatLockdown then
			AnvilSpellbook:ResizeSpellbookFrame()
		end
	end
end

function AnvilSpellbook:OnDisable()
	if not enabledCheck then
		return
	else
		AnvilSpellbookFrame:UnregisterAllEvents()
	end
end

function AnvilSpellbook_OnEvent(self, event, ...)
	if ( event == "SPELLS_CHANGED" ) then
		if ( AnvilSpellbookFrame:IsVisible() ) then
			if ( GetNumSpellTabs() < AnvilSpellbookFrame.selectedSkillLine ) then
				AnvilSpellbookFrame.selectedSkillLine = 2;
			end
			if not InCombatLockdown() then
				AnvilSpellbook:ResizeSpellbookFrame();
			end
		end
	elseif ( event == "CURRENT_SPELL_CAST_CHANGED" ) then
		if (self.castingGlyphSlot and not IsCastingGlyph()) then
			AnvilSpellbookFrame.castingGlyphSlot = nil;
			AnvilSpellbook_Update();
		end
	elseif ( event == "LEARNED_SPELL_IN_TAB" ) then
		AnvilSpellbook_Update();
		local spellID, tabNum, isGuildSpell = ...;
		local flashFrame = _G["AnvilSpellbookSkillLineTab"..tabNum.."Flash"];
		if ( AnvilSpellbookFrame.bookType == "pet" or isGuildSpell) then
			return;
		elseif ( tabNum <= GetNumSpellTabs() ) then
			if ( flashFrame ) then
				flashFrame:Show();
				AnvilSpellbookFrame.flashTabs = 1;
			end
		end
	elseif (event == "SKILL_LINES_CHANGED") and not InCombatLockdown() then
		AnvilSpellbook_UpdateProfTab();
	elseif (event == "PLAYER_GUILD_UPDATE") then
		if ( GetNumSpellTabs() < AnvilSpellbookFrame.selectedSkillLine ) then
			AnvilSpellbookFrame.selectedSkillLine = 2;
			AnvilSpellbook_Update();
		else
			AnvilSpellbook_UpdateSkillLineTabs();
		end
	elseif ( event == "PLAYER_SPECIALIZATION_CHANGED" ) then
		local unit = ...;
		if ( unit == "player" ) then
			AnvilSpellbookFrame.selectedSkillLine = 2;
			AnvilSpellbook_Update();
		end
	elseif ( event == "USE_GLYPH" ) then
		local slot = ...;
		AnvilSpellbook_OpenToPageForSlot(slot, OPEN_REASON_PENDING_GLYPH);
	elseif ( event == "CANCEL_GLYPH_CAST" ) then
		AnvilSpellbook_ClearAbilityHighlights();
		SpellFlyout:Hide();
	elseif ( event == "ACTIVATE_GLYPH" ) then
		local slot = ...;
		AnvilSpellbook_OpenToPageForSlot(slot, OPEN_REASON_ACTIVATED_GLYPH);
	elseif ( event == "PLAYER_REGEN_DISABLED" ) then
		if AnvilSpellbookFrame:IsVisible() then
			AnvilSpellbook_ToggleSpellbook(AnvilSpellbookFrame.bookType)
			postCombatLaunch = true
		end
	elseif ( event == "PLAYER_REGEN_ENABLED" ) then
		if postCombatLaunch then
			AnvilSpellbook_ToggleSpellbook(AnvilSpellbookFrame.bookType)
			postCombatLaunch = false
		end
	end
end

function AnvilSpellbook_OnShow(self)
	AnvilSpellbook_Update();

	if ( self.flashTabs ) then
		UIFrameFlash(AnvilSpellbookTabFlashFrame, 0.5, 0.5, 30, nil);
	end

	if ( AnvilSpellbookFrame.bookType == "spell" and IsCharacterNewlyBoosted() and not GetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_BOOSTED_SPELL_BOOK) ) then
		local spellSlot;
		for i = 1, spellbookSize do
			local spellBtn = _G["AnvilSpellButton" .. i];
			local slotType = select(2,AnvilSpellbook_GetSpellbookSlot(spellBtn));
			if (slotType == "FUTURESPELL") then
				if ( not spellSlot or spellBtn:GetID() < spellSlot:GetID() ) then
					spellSlot = spellBtn;
				end
			end
		end

		if ( spellSlot ) then
			SpellLockedTooltip:Show();
			SpellLockedTooltip:SetPoint("LEFT", spellSlot, "RIGHT", 16, 0);
		else
			SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_BOOSTED_SPELL_BOOK, true);
		end
	else
		SpellLockedTooltip:Hide();
	end
end

function AnvilSpellbook_OnHide(self)
	HelpPlate_Hide();
	SpellBookFrame_PlayCloseSound();

	UIFrameFlashStop(AnvilSpellbookTabFlashFrame);

	for i=1, 8 do
		_G["AnvilSpellbookSkillLineTab"..i.."Flash"]:Hide();
	end

	MultiActionBar_HideAllGrids();
	SpellLockedTooltip:Hide();

	UpdateMicroButtons();
end

function AnvilSpellbook_ToggleSpellbook(bookType)
	if not Anvil.db.global.spellbook then
		return
	elseif not InCombatLockdown() then
		SpellBookFrameCloseButton:Click()
		if ( (not HasPetSpells() or not PetHasSpellbook()) and bookType == "pet" ) then
			return;
		end

		local isShown = AnvilSpellbookFrame:IsVisible();
		if ( isShown and (AnvilSpellbookFrame.bookType == bookType) ) then
			AnvilSpellbookFrame:Hide();
		return;
		elseif isShown then
			AnvilSpellbookFrame.bookType = bookType;
			AnvilSpellbook_Update();
		else	
			AnvilSpellbookFrame.bookType = bookType;		
			AnvilSpellbookFrame:Show();
		end
	end
end

function AnvilSpellbook_ClearAbilityHighlights()
	for i = 1, spellbookSize do
		local button = _G["AnvilSpellButton"..i];
		button.AbilityHighlightAnim:Stop();
		button.AbilityHighlight:Hide();
	end
end

function AnvilSpellbook_GetCurrentPage()
	local currentPage, maxPages;
	local numPetSpells = HasPetSpells() or 0;
	if ( AnvilSpellbookFrame.bookType == "pet" ) then
		currentPage = ANVILSPELLBOOK_PAGENUMBERS["pet"];
		maxPages = ceil(numPetSpells/spellbookSize);
	elseif ( AnvilSpellbookFrame.bookType == "spell") then
		currentPage = ANVILSPELLBOOK_PAGENUMBERS[AnvilSpellbookFrame.selectedSkillLine];
		local _, _, _, numSlots = GetSpellTabInfo(AnvilSpellbookFrame.selectedSkillLine);
		maxPages = ceil(numSlots/spellbookSize);
	end
	return currentPage, maxPages;
end

function AnvilSpellbook_Update()
	AnvilSpellbookTabButton3:Hide();
	AnvilSpellbookTabButton4:Hide();
	AnvilSpellbookTabButton5:Hide();	

	AnvilSpellbookTabButton1:Show();
	AnvilSpellbookTabButton1.bookType = "spell";
	AnvilSpellbookTabButton1.binding = "TOGGLESPELLBOOK";
	AnvilSpellbookTabButton1:SetText(AnvilSpellbookInfo["spell"].title);
	AnvilSpellbookTabButton2:Show();
	AnvilSpellbookTabButton2.bookType = "professions";	
	AnvilSpellbookTabButton2:SetText(AnvilSpellbookInfo["professions"].title);
	AnvilSpellbookTabButton2.binding = "TOGGLEPROFESSIONBOOK";
	local tabIndex = 3;

	local hasPetSpells, petToken = HasPetSpells();
	AnvilSpellbookFrame.petTitle = nil;
	if ( hasPetSpells and PetHasSpellbook() ) then
		AnvilSpellbookFrame.petTitle = _G["PET_TYPE_"..petToken];
		local nextTab = _G["AnvilSpellbookTabButton"..tabIndex];
		nextTab:Show();
		nextTab.bookType = "pet";		
		nextTab.binding = "TOGGLEPETBOOK";
		nextTab:SetText(AnvilSpellbookInfo["pet"].title);
		tabIndex = tabIndex+1;
	elseif (AnvilSpellbookFrame.bookType == "pet") then
		AnvilSpellbookFrame.bookType = _G["AnvilSpellbookTabButton"..tabIndex-1].bookType;
	end

	local level = UnitLevel("player");

	for i=1, 5 do
		local tab = _G["AnvilSpellbookTabButton"..i];
		PanelTemplates_TabResize(tab, 0, nil, 40);
		if ( tab.bookType == AnvilSpellbookFrame.bookType ) then
			PanelTemplates_SelectTab(tab);
			AnvilSpellbookFrame.currentTab = tab;	
		else
			PanelTemplates_DeselectTab(tab);
		end
	end

	if (AnvilSpellbookFrame.bookType and not InCombatLockdown()) then
		for i, frame in ipairs(AnvilSpellbookFrames) do
			local found = false;
			for j,frame2 in ipairs(AnvilSpellbookInfo[AnvilSpellbookFrame.bookType].showFrames) do
				if (frame == frame2) then
					_G[frame]:Show();
					found = true;
					break;
				end
			end
			if (found == false) then
				_G[frame]:Hide();
			end
		end
		AnvilSpellbookFrameTitleText:SetText(AnvilSpellbookInfo[AnvilSpellbookFrame.bookType].title);
		local tabUpdate = AnvilSpellbookInfo[AnvilSpellbookFrame.bookType].updateFunc;
		if(tabUpdate) then
			tabUpdate()
		end
	end
	if not InCombatLockdown() then
		AnvilSpellbook:ResizeSpellbookFrame()
	end
end

function AnvilSpellbook_UpdateSpells()
	if not InCombatLockdown() then
		AnvilSpellbook_ButtonFactory()
	end
end

function AnvilSpellbook_GetSpellbookSlot(spellButton)
	local id = spellButton:GetID()
	if ( AnvilSpellbookFrame.bookType == "professions") then
		return id + spellButton:GetParent().spellOffset;
	elseif ( AnvilSpellbookFrame.bookType == "pet" ) then
		local slot = id + (spellbookSize * (ANVILSPELLBOOK_PAGENUMBERS["pet"] - 1));
		local slotType, slotID = GetSpellBookItemInfo(slot, AnvilSpellbookFrame.bookType);
		return slot, slotType, slotID;
	else
		local relativeSlot = id + ( spellbookSize * (ANVILSPELLBOOK_PAGENUMBERS[AnvilSpellbookFrame.selectedSkillLine] - 1));
		if ( AnvilSpellbookFrame.selectedSkillLineNumSlots and relativeSlot <= AnvilSpellbookFrame.selectedSkillLineNumSlots) then
			local slot = AnvilSpellbookFrame.selectedSkillLineOffset + relativeSlot;
			local slotType, slotID = GetSpellBookItemInfo(slot, AnvilSpellbookFrame.bookType);
			return slot, slotType, slotID;
		else
			return nil, nil;
		end
	end
end

function AnvilSpellbookSkillLineTab_OnClick(self)
	local id = self:GetID();
	if ( AnvilSpellbookFrame.selectedSkillLine ~= id ) then
		PlaySound(SOUNDKIT.IG_ABILITY_PAGE_TURN);
		AnvilSpellbookFrame.selectedSkillLine = id;
		AnvilSpellbook_Update();
	else
		self:SetChecked(true);
	end

	if ( self ) then
		local tabFlash = _G[self:GetName().."Flash"];
		if ( tabFlash ) then
			tabFlash:Hide();
		end
	end
end

function AnvilSpellbook_ButtonFactory()
	for i = 1, spellbookSize do
		if (not _G["AnvilSpellButton" .. i]) then
			_G["AnvilSpellButton" .. i] = CreateFrame("CheckButton", "AnvilSpellButton" .. i, AnvilSpellbookSpellIconsFrame, "SecureActionButtonTemplate, AnvilSpellButtonTemplate")
		end
		_G["AnvilSpellButton" .. i]:SetID(i)
		_G["AnvilSpellButton" .. i]:RegisterForDrag("LeftButton")
		_G["AnvilSpellButton" .. i]:Show()
		AnvilSpellButton_UpdateButton(_G["AnvilSpellButton" .. i])
	end
	for i=1, Anvil.db.global.spellbookHeight do
		for j=1, Anvil.db.global.spellbookWidth do
			_G["AnvilSpellButton"..(i-1)*(Anvil.db.global.spellbookWidth)+j]:SetPoint("TOPLEFT", AnvilSpellbookSpellIconsFrame, "TOPLEFT", (j-1)*216+12, -(i-1)*60-12)
		end
	end
end

function AnvilSpellbook:ResizeSpellbookFrame()
	spellbookSize = Anvil.db.global.spellbookWidth*Anvil.db.global.spellbookHeight
	if (Anvil.db.global.spellbookHeight<7 and AnvilSpellbookFrame.bookType == "professions") then
		AnvilSpellbookFrame:SetSize(Anvil.db.global.spellbookWidth*216 + 24, 552)
		AnvilPrimaryProfession1:SetPoint("CENTER", AnvilSpellbookFrame, "CENTER", 0, 168)
	else
		AnvilSpellbookFrame:SetSize(Anvil.db.global.spellbookWidth*216 + 18, Anvil.db.global.spellbookHeight*60 + 112)
		AnvilPrimaryProfession1:SetPoint("CENTER", AnvilSpellbookFrame, "CENTER", 0, Anvil.db.global.spellbookHeight*8+112)
	end
	AnvilSpellbookFrame:SetScale(Anvil.db.global.spellbookScale)
	local pageNum = AnvilSpellbook_GetCurrentPage()
	if AnvilSpellbookFrame.bookType == "spell" or AnvilSpellbookFrame.bookType == "pet" then
		local pageNum = AnvilSpellbook_GetCurrentPage()
		_, _, offset, numSpells = GetSpellTabInfo(AnvilSpellbookFrame.selectedSkillLine)
		AnvilSpellbook_UpdateSpells()
		AnvilSpellbook_UpdatePages()
		for i = 1, spellbookSize do
			if ((AnvilSpellbookFrame.selectedSkillLine==1 or AnvilSpellbookFrame.selectedSkillLine==2) and (i+offset+(pageNum-1)*spellbookSize)<=(offset+numSpells) and AnvilSpellbookFrame.bookType=="spell") then
				local spellType, flyoutIndex = GetSpellBookItemInfo(i+offset+((pageNum-1)*spellbookSize), "spell")
				local name, _, _, _, _, _, spellId = GetSpellInfo(i+offset+((pageNum-1)*spellbookSize), "spell")
				if spellType=="FLYOUT" then
					_G["AnvilSpellButton" .. i].action = 0
					_G["AnvilSpellButton" .. i]:SetAttribute("type1", "flyout")
					_G["AnvilSpellButton" .. i]:SetAttribute("type2", "flyout")
					_G["AnvilSpellButton" .. i]:SetAttribute("spell", flyoutIndex)
					_G["AnvilSpellButton" .. i]:SetAttribute("flyoutDirection", "RIGHT")
					_G["AnvilSpellButton" .. i]:SetScript("OnDragStart", function()
						PickupSpellBookItem(i+offset+((pageNum-1)*spellbookSize), "spell")
					end)
				elseif spellType=="SPELL" and not IsPassiveSpell(spellId) then
					_G["AnvilSpellButton" .. i]:SetAttribute("type1", "spell")
					_G["AnvilSpellButton" .. i]:SetAttribute("type2", "spell")
					_G["AnvilSpellButton" .. i]:SetAttribute("spell", name)
					_G["AnvilSpellButton" .. i]:SetScript("OnDragStart", function()
						PickupSpell(spellId)
					end)
				else
					_G["AnvilSpellButton" .. i]:SetAttribute("type1", "noop")
					_G["AnvilSpellButton" .. i]:SetAttribute("type2", "noop")
					_G["AnvilSpellButton" .. i]:SetScript("OnDragStart", function()	end)
				end
				_G["AnvilSpellButton" .. i]:RegisterForClicks("LeftButtonUp", "RightButtonUp")
			elseif AnvilSpellbookFrame.bookType == "pet" then
				local name = GetSpellInfo(i+((pageNum-1)*spellbookSize), "pet")
				_G["AnvilSpellButton" .. i]:SetAttribute("type1", "spell")
				_G["AnvilSpellButton" .. i]:SetAttribute("type2", "spell")
-- TODO: This doesn't work for basic function like Attack, Passive, etc.  Since it calls CastSpellByName and the "pet" attribute calls it off the pet action bar, it may not be doable
				_G["AnvilSpellButton" .. i]:SetAttribute("spell", name)
				_G["AnvilSpellButton" .. i]:SetScript("OnDragStart", function()
					PickupSpellBookItem(i+((pageNum-1)*spellbookSize), "pet")
				end)
			else
				_G["AnvilSpellButton" .. i]:SetAttribute("type1", "noop")
				_G["AnvilSpellButton" .. i]:SetAttribute("type2", "noop")
				_G["AnvilSpellButton" .. i]:SetScript("OnDragStart", function()	end)
			end
		end
	end
	for i = spellbookSize + 1, 100 do
		if _G["AnvilSpellButton"..i] then
			_G["AnvilSpellButton"..i]:Hide()
		end
	end
	Anvil:MakeMovable("spellbookIsMovable", "AnvilSpellbookFrame")
end

function AnvilSpellbook_UpdatePages()
	local currentPage, maxPages = AnvilSpellbook_GetCurrentPage();
	if ( maxPages == nil or maxPages == 0 ) then
		return;
	end
	if ( currentPage > maxPages ) then
		if (AnvilSpellbookFrame.bookType == "spell" ) then
			ANVILSPELLBOOK_PAGENUMBERS[AnvilSpellbookFrame.selectedSkillLine] = maxPages;
		else
			ANVILSPELLBOOK_PAGENUMBERS[AnvilSpellbookFrame.bookType] = maxPages;
		end
		currentPage = maxPages;
		if ( currentPage == 1 ) then
			AnvilSpellbookPrevPageButton:Disable();
		else
			AnvilSpellbookPrevPageButton:Enable();
		end
		if ( currentPage == maxPages ) then
			AnvilSpellbookNextPageButton:Disable();
		else
			AnvilSpellbookNextPageButton:Enable();
		end
	end
	if ( currentPage == 1 ) then
		AnvilSpellbookPrevPageButton:Disable();
	else
		AnvilSpellbookPrevPageButton:Enable();
	end
	if ( currentPage == maxPages ) then
		AnvilSpellbookNextPageButton:Disable();
	else
		AnvilSpellbookNextPageButton:Enable();
	end
	AnvilSpellbookPageText:SetFormattedText(COLLECTION_PAGE_NUMBER, currentPage, maxPages);
end

function AnvilSpellbook_UpdateSkillLineTabs()
	for i=1, 8 do
		local skillLineTab = _G["AnvilSpellbookSkillLineTab"..i];
		local prevTab = _G["AnvilSpellbookSkillLineTab"..i-1];
		if ( i <= GetNumSpellTabs() and AnvilSpellbookFrame.bookType == "spell" ) then
			local name, texture, _, _, isGuild, offSpecID, shouldHide = GetSpellTabInfo(i);
			if ( shouldHide ) then
				_G["AnvilSpellbookSkillLineTab"..i.."Flash"]:Hide();
				skillLineTab:Hide();
			else
				local isOffSpec = (offSpecID ~= 0);
				skillLineTab:SetNormalTexture(texture);
				skillLineTab.tooltip = name;
				skillLineTab:Show();
				skillLineTab.isOffSpec = isOffSpec;
				if(texture) then
					skillLineTab:GetNormalTexture():SetDesaturated(isOffSpec);
				end

				if (prevTab) then
					if (isGuild) then
						skillLineTab:SetPoint("TOPLEFT", prevTab, "BOTTOMLEFT", 0, -46);
					elseif (isOffSpec and not prevTab.isOffSpec) then
						skillLineTab:SetPoint("TOPLEFT", prevTab, "BOTTOMLEFT", 0, -40);
					else
						skillLineTab:SetPoint("TOPLEFT", prevTab, "BOTTOMLEFT", 0, -17);
					end
				end

				if (isGuild) then
					skillLineTab:SetNormalTexture("Interface\\SpellBook\\GuildSpellbooktabBG");
					skillLineTab.TabardEmblem:Show();
					skillLineTab.TabardIconFrame:Show();
					SetLargeGuildTabardTextures("player", skillLineTab.TabardEmblem, skillLineTab:GetNormalTexture(), skillLineTab.TabardIconFrame);
				else
					skillLineTab.TabardEmblem:Hide();
					skillLineTab.TabardIconFrame:Hide();
				end

				if ( AnvilSpellbookFrame.selectedSkillLine == i ) then
					skillLineTab:SetChecked(true);
				else
					skillLineTab:SetChecked(false);
				end
			end
		else
			_G["AnvilSpellbookSkillLineTab"..i.."Flash"]:Hide();
			skillLineTab:Hide();
		end
	end
end

function AnvilSpellbook_UpdatePlayerTab()
	local _, _, offset, numSlots = GetSpellTabInfo(AnvilSpellbookFrame.selectedSkillLine);
	AnvilSpellbookFrame.selectedSkillLineOffset = offset;
	AnvilSpellbookFrame.selectedSkillLineNumSlots = numSlots;

	AnvilSpellbook_UpdatePages();
	AnvilSpellbook_UpdateSkillLineTabs();
	AnvilSpellbook_UpdateSpells();
end

function AnvilSpellbook_UpdatePetTab()
	AnvilSpellbook_UpdatePages();
	AnvilSpellbook_UpdateSpells();
end

function AnvilSpellButton_OnEvent(self, event, ...)
	if ( event == "SPELLS_CHANGED" or event == "UPDATE_SHAPESHIFT_FORM" ) then
		AnvilSpellButton_UpdateButton(self);
	elseif ( event == "SPELL_UPDATE_COOLDOWN" ) then
		AnvilSpellButton_UpdateCooldown(self);
		if ( GameTooltip:GetOwner() == self ) then
			AnvilSpellButton_OnEnter(self);
		end
	elseif ( event == "CURRENT_SPELL_CAST_CHANGED" ) then
		AnvilSpellButton_UpdateSelection(self);
	elseif ( event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE" or event == "ARCHAEOLOGY_CLOSED" ) then
		AnvilSpellButton_UpdateSelection(self);
	elseif ( event == "PET_BAR_UPDATE" ) then
		if ( AnvilSpellbookFrame.bookType == "pet" ) then
			AnvilSpellButton_UpdateButton(self);
		end
	elseif ( event == "CURSOR_UPDATE" ) then
		if ( self.spellGrabbed ) then
			AnvilSpellButton_UpdateButton(self);
			self.spellGrabbed = false;
		end
	elseif ( event == "ACTIONBAR_SLOT_CHANGED" ) then
		local slot, slotType, slotID = AnvilSpellbook_GetSpellbookSlot(self);
		if ( not slot ) then
			return;
		end

		local _, actionID = GetSpellBookItemInfo(slot, AnvilSpellbookFrame.bookType);
		local onActionBar = false;
		
		if ( slotType == "SPELL" ) then
			if (FindFlyoutSlotBySpellID(actionID) > 0) then
				AnvilSpellbook_UpdateSpells();
			else
				onActionBar = C_ActionBar.IsOnBarOrSpecialBar(actionID);
			end
		elseif ( slotType == "FLYOUT" ) then
			onActionBar = C_ActionBar.HasFlyoutActionButtons(actionID);
		elseif ( slotType == "PETACTION" ) then
			onActionBar = C_ActionBar.HasPetActionButtons(actionID) or C_ActionBar.HasPetActionPetBarIndices(actionID);
		end

		if ( self.SpellHighlightTexture and self.SpellHighlightTexture:IsShown() == onActionBar ) then
			AnvilSpellButton_UpdateButton(self);
		end
	end
end

function AnvilSpellButton_OnEnter(self)
	local slot = AnvilSpellbook_GetSpellbookSlot(self);
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	if ( GameTooltip:SetSpellBookItem(slot, AnvilSpellbookFrame.bookType) ) then
		self.UpdateTooltip = AnvilSpellButton_OnEnter;
	else
		self.UpdateTooltip = nil;
	end

	if ( self.SpellHighlightTexture and self.SpellHighlightTexture:IsShown() ) then
		GameTooltip:AddLine(SPELLBOOK_SPELL_NOT_ON_ACTION_BAR, LIGHTBLUE_FONT_COLOR.r, LIGHTBLUE_FONT_COLOR.g, LIGHTBLUE_FONT_COLOR.b);
	end

	GameTooltip:Show();
end

function AnvilSpellButton_OnLeave(self)
	GameTooltip:Hide();
end

function AnvilSpellButton_UpdateButton(self)
	if AnvilSpellbookFrame.bookType == "professions" then
		AnvilSpellButton_UpdateProfessionButton(self);
		return;
	end

	if ( not AnvilSpellbookFrame.selectedSkillLine ) then
		AnvilSpellbookFrame.selectedSkillLine = 2;
	end
	local _, _, offset, numSlots, _, offSpecID = GetSpellTabInfo(AnvilSpellbookFrame.selectedSkillLine);
	AnvilSpellbookFrame.selectedSkillLineNumSlots = numSlots;
	AnvilSpellbookFrame.selectedSkillLineOffset = offset;
	local isOffSpec = (offSpecID ~= 0) and (AnvilSpellbookFrame.bookType == "spell");
	self.offSpecID = offSpecID;

	if (not self.SpellName.shadowX) then
		self.SpellName.shadowX, self.SpellName.shadowY = self.SpellName:GetShadowOffset();
	end

	local slot, slotType, slotID = AnvilSpellbook_GetSpellbookSlot(self);
	local name = self:GetName();
	local iconTexture = _G[name.."IconTexture"];
	local spellString = _G[name.."SpellName"];
	local subSpellString = _G[name.."SubSpellName"];
	local cooldown = _G[name.."Cooldown"];
	local autoCastableTexture = _G[name.."AutoCastable"];
	local slotFrame = _G[name.."SlotFrame"];

	if (SpellFlyout:IsShown() and SpellFlyout:GetParent() == self and not HasPendingGlyphCast() and not SpellFlyout.glyphActivating)  then
		SpellFlyout:Hide();
	end

	local highlightTexture = _G[name.."Highlight"];
	local texture;
	if ( slot ) then
		texture = GetSpellBookItemTexture(slot, AnvilSpellbookFrame.bookType);
	end

	if ( not texture or (strlen(texture) == 0) or (slotType == "FUTURESPELL" and IsKioskModeEnabled())) then
		iconTexture:Hide();
		spellString:Hide();
		subSpellString:Hide();
		cooldown:Hide();
		autoCastableTexture:Hide();
		AnvilSpellbook_ReleaseAutoCastShine(self.shine);
		self.shine = nil;
		highlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
		self:SetChecked(false);
		slotFrame:Hide();
		self.IconTextureBg:Hide();
		self.SeeTrainerString:Hide();
		self.RequiredLevelString:Hide();
		self.UnlearnedFrame:Hide();
		self.TrainFrame:Hide();
		self.TrainTextBackground:Hide();
		self.TrainBook:Hide();
		self.FlyoutArrow:Hide();
		self.AbilityHighlightAnim:Stop();
		self.AbilityHighlight:Hide();
		self.GlyphIcon:Hide();
		self:Disable();
		self.EmptySlot:SetDesaturated(isOffSpec);
		if self.SpellHighlightTexture then
			self.SpellHighlightTexture:Hide();
		end
		return;
	else
		self:Enable();
	end

	AnvilSpellButton_UpdateCooldown(self);

	local autoCastAllowed, autoCastEnabled = GetSpellAutocast(slot, AnvilSpellbookFrame.bookType);
	if ( autoCastAllowed ) then
		autoCastableTexture:Show();
	else
		autoCastableTexture:Hide();
	end
	if ( autoCastEnabled and not self.shine ) then
		self.shine = AnvilSpellbook_GetAutoCastShine();
		self.shine:Show();
		self.shine:SetParent(self);
		self.shine:SetPoint("CENTER", self, "CENTER");
		AutoCastShine_AutoCastStart(self.shine);
	elseif ( autoCastEnabled ) then
		self.shine:Show();
		self.shine:SetParent(self);
		self.shine:SetPoint("CENTER", self, "CENTER");
		AutoCastShine_AutoCastStart(self.shine);
	elseif ( not autoCastEnabled ) then
		AnvilSpellbook_ReleaseAutoCastShine(self.shine);
		self.shine = nil;
	end

	local spellName, subSpellName = GetSpellBookItemName(slot, AnvilSpellbookFrame.bookType);
	local isPassive = IsPassiveSpell(slot, AnvilSpellbookFrame.bookType);
	self.isPassive = isPassive;

	if (slotType == "FLYOUT") then
		SetClampedTextureRotation(self.FlyoutArrow, 90);
		self.FlyoutArrow:Show();
	else
		self.FlyoutArrow:Hide();
	end

	local specs =  {GetSpecsForSpell(slot, AnvilSpellbookFrame.bookType)};
	local specName = table.concat(specs, PLAYER_LIST_DELIMITER);
	if ( subSpellName == "" ) then
		if ( IsTalentSpell(slot, AnvilSpellbookFrame.bookType) ) then
			if ( isPassive ) then
				subSpellName = TALENT_PASSIVE
			else
				subSpellName = TALENT
			end
		elseif ( isPassive ) then
			subSpellName = SPELL_PASSIVE;
		end
	end			

	if ( subSpellName == "" ) then
		self.SpellSubName:SetHeight(6);
	else
		self.SpellSubName:SetHeight(0);
	end

	iconTexture:SetTexture(texture);
	spellString:SetText(spellName);
	subSpellString:SetText(subSpellName);
	iconTexture:Show();
	spellString:Show();
	subSpellString:Show();

	if (not (slotType == "FUTURESPELL")) then
		slotFrame:Show();
		self.UnlearnedFrame:Hide();
		self.TrainFrame:Hide();
		self.IconTextureBg:Hide();
		iconTexture:SetAlpha(1);
		iconTexture:SetDesaturated(false);
		self.RequiredLevelString:Hide();
		self.SeeTrainerString:Hide();
		self.TrainTextBackground:Hide();
		self.TrainBook:Hide();
		self.SpellName:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
		self.SpellName:SetShadowOffset(self.SpellName.shadowX, self.SpellName.shadowY);
		self.SpellName:SetPoint("LEFT", self, "RIGHT", 8, 4);
		self.SpellSubName:SetTextColor(1, 1, 1);
		local _, actionID = GetSpellBookItemInfo(slot, AnvilSpellbookFrame.bookType);
		if ( slotType == "SPELL" and not isOffSpec ) then
			if (IsSpellValidForPendingGlyph(actionID)) then
				self.AbilityHighlight:Show();
				self.AbilityHighlightAnim:Play();
			else
				self.AbilityHighlightAnim:Stop();
				self.AbilityHighlight:Hide();
			end
			if (HasAttachedGlyph(actionID) or AnvilSpellbookFrame.castingGlyphSlot == slot) then
				self.GlyphIcon:Show();
			else
				self.GlyphIcon:Hide();
			end
		else
			self.AbilityHighlightAnim:Stop();
			self.AbilityHighlight:Hide();
			self.GlyphIcon:Hide();
		end

		if self.SpellHighlightTexture then
			self.SpellHighlightTexture:Hide();
			if ( AnvilSpellbookFrame.selectedSkillLine == 2 or AnvilSpellbookFrame.bookType == "pet" ) then
				if ( slotType == "SPELL" ) then
					self.SpellHighlightTexture:SetShown(not isPassive and not C_ActionBar.IsOnBarOrSpecialBar(actionID));
				elseif ( slotType == "FLYOUT" ) then
					self.SpellHighlightTexture:SetShown(not C_ActionBar.HasFlyoutActionButtons(actionID));
				elseif ( slotType == "PETACTION" ) then
					if ( isPassive ) then
						self.SpellHighlightTexture:Hide();
					else
						local onBarSomewhere = C_ActionBar.HasPetActionButtons(actionID) or C_ActionBar.HasPetActionPetBarIndices(actionID);
						self.SpellHighlightTexture:SetShown(not onBarSomewhere);
					end
				end
			end
		end

		if ( slotType == "SPELL" and isOffSpec ) then
			local level = GetSpellLevelLearned(slotID);
			if ( level and level > UnitLevel("player") ) then
				self.RequiredLevelString:Show();
				self.RequiredLevelString:SetFormattedText(SPELLBOOK_AVAILABLE_AT, level);
				self.RequiredLevelString:SetTextColor(0.25, 0.12, 0);
			end
		end
	else
		local level = GetSpellAvailableLevel(slot, AnvilSpellbookFrame.bookType);
		slotFrame:Hide();
		self.AbilityHighlightAnim:Stop();
		self.AbilityHighlight:Hide();
		if self.SpellHighlightTexture then
			self.SpellHighlightTexture:Hide();
		end
		self.GlyphIcon:Hide();
		self.IconTextureBg:Show();
		iconTexture:SetAlpha(0.5);
		iconTexture:SetDesaturated(true);
		if (IsCharacterNewlyBoosted()) then
			self.SeeTrainerString:Hide();
			self.UnlearnedFrame:Show();
			self.TrainFrame:Hide();
			self.TrainTextBackground:Hide();
			self.TrainBook:Hide();
			self.RequiredLevelString:Show();
			self.RequiredLevelString:SetText(BOOSTED_CHAR_SPELL_TEMPLOCK);
			self.RequiredLevelString:SetTextColor(0.25, 0.12, 0);
			self.SpellName:SetTextColor(0.25, 0.12, 0);
			self.SpellSubName:SetTextColor(0.25, 0.12, 0);
			self.SpellName:SetShadowOffset(0, 0);
			self.SpellName:SetPoint("LEFT", self, "RIGHT", 8, 6);
		elseif (level and level > UnitLevel("player")) then
			self.SeeTrainerString:Hide();
			self.RequiredLevelString:Show();
			self.RequiredLevelString:SetFormattedText(SPELLBOOK_AVAILABLE_AT, level);
			self.RequiredLevelString:SetTextColor(0.25, 0.12, 0);
			self.UnlearnedFrame:Show();
			self.TrainFrame:Hide();
			self.TrainTextBackground:Hide();
			self.TrainBook:Hide();
			self.SpellName:SetTextColor(0.25, 0.12, 0);
			self.SpellSubName:SetTextColor(0.25, 0.12, 0);
			self.SpellName:SetShadowOffset(0, 0);
			self.SpellName:SetPoint("LEFT", self, "RIGHT", 8, 6);
		else
			self.SeeTrainerString:Show();
			self.RequiredLevelString:Hide();
			self.TrainFrame:Show();
			self.UnlearnedFrame:Hide();
			self.TrainTextBackground:Show();
			self.TrainBook:Show();
			self.SpellName:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			self.SpellName:SetShadowOffset(self.SpellName.shadowX, self.SpellName.shadowY);
			self.SpellName:SetPoint("LEFT", self, "RIGHT", 24, 8);
			self.SpellSubName:SetTextColor(0, 0, 0);
		end
	end

	if ( isPassive ) then
		highlightTexture:SetTexture("Interface\\Buttons\\UI-PassiveHighlight");
		slotFrame:Hide();
		self.UnlearnedFrame:Hide();
	else
		highlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
	end

	slotFrame:SetDesaturated(isOffSpec);
	self.EmptySlot:SetDesaturated(isOffSpec);
	self.FlyoutArrow:SetDesaturated(isOffSpec);
	if (isOffSpec) then
		iconTexture:SetDesaturated(isOffSpec);
		self.SpellName:SetTextColor(0.75, 0.75, 0.75);
		self.RequiredLevelString:SetTextColor(0.1, 0.1, 0.1);
		autoCastableTexture:Hide();
		AnvilSpellbook_ReleaseAutoCastShine(self.shine);
		self.shine = nil;
		self:SetChecked(false);
	else
		AnvilSpellButton_UpdateSelection(self);
	end
end

function AnvilFormatProfession(frame, index)
	if index then
		frame.missingHeader:Hide();
		frame.missingText:Hide();

		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier, specializationIndex, specializationOffset = GetProfessionInfo(index);
		frame.skillName = name;
		frame.spellOffset = spelloffset;
		frame.skillLine = skillLine;
		frame.specializationIndex = specializationIndex;
		frame.specializationOffset = specializationOffset;

		frame.statusBar:SetMinMaxValues(1,maxRank);
		frame.statusBar:SetValue(rank);

		local prof_title = "";
		for i=1,#PROFESSION_RANKS do
			local value,title = PROFESSION_RANKS[i][1], PROFESSION_RANKS[i][2]; 
			if maxRank < value then break end
				prof_title = title;
		end
		frame.rank:SetText(prof_title);

		frame.statusBar:Show();
		if rank == maxRank then
			frame.statusBar.capRight:Show();
		else
			frame.statusBar.capRight:Hide();
		end
		if ( GameLimitedMode_IsActive() ) then
			local _, _, profCap = GetRestrictedAccountData();
			if rank >= profCap then
				frame.statusBar.capped:Show();
				frame.statusBar.rankText:SetTextColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
				frame.statusBar.tooltip = RED_FONT_COLOR_CODE..CAP_REACHED_TRIAL..FONT_COLOR_CODE_CLOSE;
			else
				frame.statusBar.capped:Hide();
				frame.statusBar.rankText:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
				frame.statusBar.tooltip = nil;
			end
		end
	
		if frame.icon and texture then
			SetPortraitToTexture(frame.icon, texture);	
			frame.unlearn:Show();
		end
	
		frame.professionName:SetText(name);
			if ( rankModifier > 0 ) then
			frame.statusBar.rankText:SetFormattedText(TRADESKILL_RANK_WITH_MODIFIER, rank, rankModifier, maxRank);
		else
			frame.statusBar.rankText:SetFormattedText(TRADESKILL_RANK, rank, maxRank);
		end
			if numSpells <= 0 then		
			frame.button1:Hide();
			frame.button2:Hide();
		elseif numSpells == 1 then		
			frame.button2:Hide();
			frame.button1:Show();
			AnvilSpellButton_UpdateProfessionButton(frame.button1);		
		else
			frame.button1:Show();
			frame.button2:Show();
			AnvilSpellButton_UpdateProfessionButton(frame.button1);			
			AnvilSpellButton_UpdateProfessionButton(frame.button2);
		end
	else
		frame.missingHeader:Show();
		frame.missingText:Show();

		if frame.icon then
			SetPortraitToTexture(frame.icon, "Interface\\Icons\\INV_Scroll_04");	
			frame.unlearn:Hide();			
			frame.specialization:SetText("");
		end			
		frame.button1:Hide();
		frame.button2:Hide();
		frame.statusBar:Hide();
		frame.rank:SetText("");
		frame.professionName:SetText("");
	end		
end

function AnvilSpellbook_UpdateProfTab()
	local prof1, prof2, arch, fish, cook, firstAid = GetProfessions();
	AnvilFormatProfession(AnvilPrimaryProfession1, prof1);
	AnvilFormatProfession(AnvilPrimaryProfession2, prof2);
	AnvilFormatProfession(AnvilSecondaryProfession1, arch);
	AnvilFormatProfession(AnvilSecondaryProfession2, fish);
	AnvilFormatProfession(AnvilSecondaryProfession3, cook);
	AnvilFormatProfession(AnvilSecondaryProfession4, firstAid);
end

function AnvilSpellButton_UpdateProfessionButton(self)
	local spellIndex = self:GetID() + self:GetParent().spellOffset;
	local texture = GetSpellBookItemTexture(spellIndex, AnvilSpellbookFrame.bookType);
	local spellName, subSpellName = GetSpellBookItemName(spellIndex, AnvilSpellbookFrame.bookType);
	local isPassive = IsPassiveSpell(spellIndex, AnvilSpellbookFrame.bookType);
	if ( isPassive ) then
		self.highlightTexture:SetTexture("Interface\\Buttons\\UI-PassiveHighlight");
		self.spellString:SetTextColor(PASSIVE_SPELL_FONT_COLOR.r, PASSIVE_SPELL_FONT_COLOR.g, PASSIVE_SPELL_FONT_COLOR.b);
	else
		self.highlightTexture:SetTexture("Interface\\Buttons\\ButtonHilight-Square");
		self.spellString:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
	end

	self.iconTexture:SetTexture(texture);
	local start, duration, enable = GetSpellCooldown(spellIndex, AnvilSpellbookFrame.bookType);
	CooldownFrame_Set(self.cooldown, start, duration, enable);
	if ( enable == 1 ) then
		self.iconTexture:SetVertexColor(1.0, 1.0, 1.0);
	else
		self.iconTexture:SetVertexColor(0.4, 0.4, 0.4);
	end

	if ( self:GetParent().specializationIndex >= 0 and self:GetID() == self:GetParent().specializationOffset) then
		self.unlearn:Show();
	else
		self.unlearn:Hide();
	end

	self.spellString:SetText(spellName);
	self.subSpellString:SetText(subSpellName);	
	self.iconTexture:SetTexture(texture);
	self:SetAttribute("type", "professionClick")
	self:SetAttribute("_professionClick", AnvilProfessionButton_OnClick)
	self:RegisterForDrag("LeftButton")
	self:SetScript("OnDragStart", function() PickupSpellBookItem(spellName) end)
	AnvilSpellButton_UpdateSelection(self);
end

function AnvilProfessionButton_OnClick(self)
	local spellIndex = self:GetID() + self:GetParent().spellOffset;
	local spellName = GetSpellBookItemName(spellIndex, AnvilSpellbookFrame.bookType);
	CastSpellByName(spellName)
	AnvilSpellbookFrameCloseButton:Click()
end

function AnvilSpellButton_UpdateSelection(self)
	if AnvilSpellbookFrame.bookType == "professions" then
		local slot = AnvilSpellbook_GetSpellbookSlot(self);
		if ( slot and IsSelectedSpellBookItem(slot, AnvilSpellbookFrame.bookType) ) then
			self:SetChecked(true);
		else
			self:SetChecked(false);
		end
	end
end

function AnvilSpellButton_UpdateCooldown(self)
	local cooldown = self.cooldown;
	local slot, slotType = AnvilSpellbook_GetSpellbookSlot(self);
	if (slot) then
		local start, duration, enable, modRate = GetSpellCooldown(slot, AnvilSpellbookFrame.bookType);
		if (cooldown and start and duration) then
			if (enable) then
				cooldown:Hide();
			else
				cooldown:Show();
			end
			CooldownFrame_Set(cooldown, start, duration, enable, false, modRate);
		else
			cooldown:Hide();
		end
	end
end

function AnvilSpellbookPrevPageButton_OnClick()
	local pageNum = AnvilSpellbook_GetCurrentPage() - 1;
	PlaySound(SOUNDKIT.IG_ABILITY_PAGE_TURN);
	if ( AnvilSpellbookFrame.bookType == "spell" ) then
		ANVILSPELLBOOK_PAGENUMBERS[AnvilSpellbookFrame.selectedSkillLine] = pageNum;
	else
		ANVILSPELLBOOK_PAGENUMBERS[AnvilSpellbookFrame.bookType] = pageNum;
	end
	AnvilSpellbook_Update();
end

function AnvilSpellbookNextPageButton_OnClick()
	local pageNum = AnvilSpellbook_GetCurrentPage() + 1;
	PlaySound(SOUNDKIT.IG_ABILITY_PAGE_TURN);
	if ( AnvilSpellbookFrame.bookType == "spell" ) then
		ANVILSPELLBOOK_PAGENUMBERS[AnvilSpellbookFrame.selectedSkillLine] = pageNum;
	else
		ANVILSPELLBOOK_PAGENUMBERS[AnvilSpellbookFrame.bookType] = pageNum;
	end
	AnvilSpellbook_Update();
end

function AnvilSpellbook_OnMouseWheel(self, value, scrollBar)
	if(AnvilSpellbookFrame.bookType ~= "spell") then
		return;
	end

	local currentPage, maxPages = AnvilSpellbook_GetCurrentPage();

	if(value > 0) then
		if(currentPage > 1) then
			AnvilSpellbookPrevPageButton_OnClick()
		end
	else 
		if(currentPage < maxPages) then
			AnvilSpellbookNextPageButton_OnClick()
		end
	end
end

function AnvilSpellbookFrameTabButton_OnClick(self)
	self:Disable();
	if AnvilSpellbookFrame.currentTab then
		AnvilSpellbookFrame.currentTab:Enable();
	end
	AnvilSpellbookFrame.currentTab = self;
	AnvilSpellbook_ToggleSpellbook(self.bookType);
end

local maxShines = 1;
local shineGet = {}

function AnvilSpellbook_GetAutoCastShine ()
	local shine = shineGet[1];

	if ( shine ) then
		tremove(shineGet, 1);
	else
		shine = CreateFrame("FRAME", "AnvilAutocastShine" .. maxShines, AnvilSpellbookFrame, "SpellBookShineTemplate");
		maxShines = maxShines + 1;
	end
	return shine;
end

function AnvilSpellbook_ReleaseAutoCastShine (shine)
	if ( not shine ) then
		return;
	end
	
	shine:Hide();
	AutoCastShine_AutoCastStop(shine);
	tinsert(shineGet, shine);
end
