AnvilToys = Anvil:NewModule("AnvilToys", "AceConsole-3.0", "AceEvent-3.0")
local _G = _G
local anvilToysPerPage, toyID

function AnvilToys:OnEnable()
	if not Anvil.db.global.collections then
		self:OnDisable()
	else
		AnvilToyBox.currentPage = 1
		AnvilToyBox.maxPages = 1
		enabledCheck = true
		AnvilToyBox.firstCollectedToyID = 0;
		AnvilToyBox.mostRecentCollectedToyID = UIParent.mostRecentCollectedToyID or nil;
		AnvilToyBox.newToys = UIParent.newToys or {};
		self:ResizeToyBox()
		AnvilToyBox_UpdatePages();
		ToyBox_UpdateProgressBar(AnvilToyBox);

		UIDropDownMenu_Initialize(AnvilToyBox.toyOptionsMenu, AnvilToyBoxOptionsMenu_Init, "MENU");

		AnvilToys:RegisterEvent("TOYS_UPDATED", AnvilToyBox_OnEvent);

		AnvilToyBox.OnPageChanged = function(userAction)
			self:ResizeToyBox();
		end
	end
end

function AnvilToys:OnDisable()
	if not enabledCheck then
		return
	else
		self:UnregisterAllEvents()
	end
end

function AnvilToyBox_OnEvent(self, event, itemID, new)
	if ( event == "TOYS_UPDATED" ) then
		if (new) then
			self.mostRecentCollectedToyID = itemID;
			if ( not AnvilCollectionsJournal:IsShown() ) then
				AnvilCollectionsJournal_SetTab(AnvilCollectionsJournal, 3);
			end
		end

		AnvilToyBox_UpdatePages();
		ToyBox_UpdateProgressBar(self);
		AnvilToys:ResizeToyBox();

		if (new) then
			self.newToys[itemID] = true;
		end
	end
end

function AnvilToyBox_OnShow(self)
	SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TOYBOX, true);

	if(C_ToyBox.HasFavorites()) then 
		SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TOYBOX_FAVORITE, true);
		self.favoriteHelpBox:Hide();
	end

	SetPortraitToTexture(AnvilCollectionsJournalPortrait, "Interface\\Icons\\Trade_Archaeology_ChestofTinyGlassAnimals");
	C_ToyBox.ForceToyRefilter();

	AnvilToyBox_UpdatePages();	
	ToyBox_UpdateProgressBar(self);
	AnvilToys:ResizeToyBox();
end

function AnvilToyBox_FindPageForToyID(toyID)
	for i = 1, C_ToyBox.GetNumFilteredToys() do
		if C_ToyBox.GetToyFromIndex(i) == toyID then
			return math.floor((i - 1) / anvilToysPerPage) + 1;
		end
	end

	return nil;
end

function AnvilToyBox_OnMouseWheel(self, value)
	SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TOYBOX_MOUSEWHEEL_PAGING, true);
	self.mousewheelPagingHelpBox:Hide();
	AnvilToyBox.PagingFrame:OnMouseWheel(value);
end

function AnvilToyBoxOptionsMenu_Init(self, level)
	local info = UIDropDownMenu_CreateInfo();
	info.notCheckable = true;
	info.disabled = nil;

	local isFavorite = AnvilToyBox.menuItemID and C_ToyBox.GetIsFavorite(ToyBox.menuItemID);

	if (isFavorite) then
		info.text = BATTLE_PET_UNFAVORITE;
		info.func = function() 
			C_ToyBox.SetIsFavorite(AnvilToyBox.menuItemID, false);
		end
	else
		info.text = BATTLE_PET_FAVORITE;
		info.func = function() 
			C_ToyBox.SetIsFavorite(AnvilToyBox.menuItemID, true);
			SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TOYBOX_FAVORITE, true);
			AnvilToyBox.favoriteHelpBox:Hide();
		end
	end

	UIDropDownMenu_AddButton(info, level);
	info.disabled = nil;
	
	info.text = CANCEL
	info.func = nil
	UIDropDownMenu_AddButton(info, level)
end

function AnvilToyBox_ShowToyDropdown(itemID, anchorTo, offsetX, offsetY)	
	AnvilToyBox.menuItemID = itemID;
	ToggleDropDownMenu(1, nil, AnvilToyBox.toyOptionsMenu, anchorTo, offsetX, offsetY);
	PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
end

function AnvilToyBox_HideToyDropdown()
	if (UIDropDownMenu_GetCurrentDropDown() == AnvilToyBox.toyOptionsMenu) then
		HideDropDownMenu(1);
	end
end

function AnvilToySpellButton_OnShow(self)
	self:RegisterEvent("TOYS_UPDATED");
	AnvilCollectionsSpellButton_OnShow(self);
end

function AnvilToySpellButton_OnHide(self)
	AnvilCollectionsSpellButton_OnHide(self);
	self:UnregisterEvent("TOYS_UPDATED");	
end

function AnvilToySpellButton_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
	if ( GameTooltip:SetToyByItemID(self.itemID) ) then
		self.UpdateTooltip = AnvilToySpellButton_OnEnter;
	else
		self.UpdateTooltip = nil;
	end

	if(AnvilToyBox.newToys[self.itemID] ~= nil) then
		AnvilToyBox.newToys[self.itemID] = nil;
		AnvilToySpellButton_UpdateButton(self);
	end
end

function AnvilToys:ResizeToyBox()
	AnvilToyBox.favoriteHelpBox:Hide();
	anvilToysPerPage = 16+Anvil.db.global.collectionsWidth*8
	wipe(AnvilToyBox.iconsFrame)
	for i=1, 56 do
		if i<=anvilToysPerPage then
			if not _G["anvilToyButton"..i] then
				_G["anvilToyButton"..i] = CreateFrame("CheckButton", "anvilToyButton"..i, AnvilToyBox, "AnvilToySpellButtonTemplate, SecureActionButtonTemplate")
			end
			_G["anvilToyButton"..i].toyID = C_ToyBox.GetToyFromIndex(i+((AnvilToyBox.PagingFrame:GetCurrentPage()-1)*(anvilToysPerPage)))
			_G["anvilToyButton"..i]:SetID(i)
			_G["anvilToyButton"..i]:Show()
			_G["anvilToyButton"..i]:SetAttribute("type1", "toy")
			_G["anvilToyButton"..i]:SetAttribute("toy", _G["anvilToyButton"..i].toyID)
			_G["anvilToyButton"..i]:SetAttribute("shift-type1", "favoritesClick")
			_G["anvilToyButton"..i]:SetAttribute("type2", "favoritesClick")
			_G["anvilToyButton"..i]:SetAttribute("_favoritesClick", AnvilToySpellButton_OnClick)
			_G["anvilToyButton"..i]:SetScript("OnDragStart", function()
				C_ToyBox.PickupToyBoxItem(_G["anvilToyButton"..i].toyID)
			end)
			tinsert(AnvilToyBox.iconsFrame, _G["anvilToyButton"..i])
		elseif _G["anvilToyButton"..i] then
			_G["anvilToyButton"..i]:Hide()
		end
	end
	for i=1, 8 do
		for j=1, 2+Anvil.db.global.collectionsWidth do
			_G["anvilToyButton"..(i-1)*(2+Anvil.db.global.collectionsWidth)+j]:SetPoint("TOPLEFT", AnvilToyBox, "TOPLEFT", (j-1)*236+18, -i*64-12)
		end
	end
	AnvilToyBox_UpdatePages()
	AnvilToyBox_UpdateButtons()
end

function AnvilToySpellButton_UpdateButton(self)
	local itemIndex = (AnvilToyBox.PagingFrame:GetCurrentPage() - 1) * anvilToysPerPage + self:GetID();
	self.itemID = C_ToyBox.GetToyFromIndex(itemIndex);
	local toyString = self.name;
	local toyNewString = self.new;
	local toyNewGlow = self.newGlow;
	local iconTexture = self.iconTexture;
	local iconTextureUncollected = self.iconTextureUncollected;
	local slotFrameCollected = self.slotFrameCollected;
	local slotFrameUncollected = self.slotFrameUncollected;
	local slotFrameUncollectedInnerGlow = self.slotFrameUncollectedInnerGlow;
	local iconFavoriteTexture = self.cooldownWrapper.slotFavorite;
	if (self.itemID == -1) then	
		self:Hide();		
		return;
	end
	self:Show();
	local itemID, toyName, icon = C_ToyBox.GetToyInfo(self.itemID);
	if toyName then
		if string.len(toyName) == 0 then
			toyName = itemID;
		end
	end
	iconTexture:SetTexture(icon);
	iconTextureUncollected:SetTexture(icon);
	iconTextureUncollected:SetDesaturated(true);
	toyString:SetText(toyName);	
	toyString:Show();

	if (AnvilToyBox.newToys[self.itemID] ~= nil) then
		toyNewString:Show();
		toyNewGlow:Show();
	else
		toyNewString:Hide();
		toyNewGlow:Hide();
	end

	if (C_ToyBox.GetIsFavorite(itemID)) then
		iconFavoriteTexture:Show();
	else
		iconFavoriteTexture:Hide();
	end

	if (PlayerHasToy(self.itemID)) then
		iconTexture:Show();
		iconTextureUncollected:Hide();
		toyString:SetTextColor(1, 0.82, 0, 1);
		toyString:SetShadowColor(0, 0, 0, 1);
		slotFrameCollected:Show();
		slotFrameUncollected:Hide();
		slotFrameUncollectedInnerGlow:Hide();

		if(AnvilToyBox.firstCollectedToyID == 0) then
			AnvilToyBox.firstCollectedToyID = self.itemID;
		end

		if (AnvilToyBox.firstCollectedToyID == self.itemID and not AnvilToyBox.favoriteHelpBox:IsVisible() and not GetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TOYBOX_FAVORITE)) then
			AnvilToyBox.favoriteHelpBox:Show();
			AnvilToyBox.favoriteHelpBox:SetPoint("TOPLEFT", self, "BOTTOMLEFT", -5, -20);
		end
	else
		iconTexture:Hide();
		iconTextureUncollected:Show();
		toyString:SetTextColor(0.33, 0.27, 0.20, 1);
		toyString:SetShadowColor(0, 0, 0, 0.33);
		slotFrameCollected:Hide();
		slotFrameUncollected:Show();		
		slotFrameUncollectedInnerGlow:Show();
	end

	AnvilCollectionsSpellButton_UpdateCooldown(self);
end

function AnvilToySpellButton_OnClick(self, button)
	if ( IsModifiedClick("CHATLINK") ) then
		local itemLink = C_ToyBox.GetToyLink(self.itemID);
		if ( itemLink ) then
			ChatEdit_InsertLink(itemLink);
		end
	elseif ( button ~= "LeftButton" ) then
		if (PlayerHasToy(self.itemID)) then
			ToyBox_ShowToyDropdown(self.itemID, self, 0, 0);
		end
	else
		UseToy(self.itemID);
	end
end

function AnvilToyBox_UpdateButtons()
	ToyBox.favoriteHelpBox:Hide();
	for i = 1, anvilToysPerPage do
	    local button = _G["anvilToyButton"..i];
		AnvilToySpellButton_UpdateButton(button);
	end	
end

function AnvilToyBox_UpdatePages()
	local maxPages = 1 + math.floor( math.max((C_ToyBox.GetNumFilteredToys() - 1), 0) / anvilToysPerPage);
	AnvilToyBox.PagingFrame:SetMaxPages(maxPages)
	if AnvilToyBox.mostRecentCollectedToyID then
		local toyPage = AnvilToyBox_FindPageForToyID(AnvilToyBox.mostRecentCollectedToyID);
		if toyPage then
			AnvilToyBox.PagingFrame:SetCurrentPage(toyPage);
		end
		AnvilToyBox.mostRecentCollectedToyID = nil;
	end
end

function AnvilToyBox_OnSearchTextChanged(self)
	SearchBoxTemplate_OnTextChanged(self);
	local oldText = AnvilToyBox.searchString;
	AnvilToyBox.searchString = self:GetText();

	if ( oldText ~= AnvilToyBox.searchString ) then		
		AnvilToyBox.firstCollectedToyID = 0;
		C_ToyBox.SetFilterString(AnvilToyBox.searchString);
		AnvilToyBox_UpdatePages();
		AnvilToys:ResizeToyBox();
	end
end

function AnvilToyBoxFilterDropDown_OnLoad(self)
	UIDropDownMenu_Initialize(self, AnvilToyBoxFilterDropDown_Initialize, "MENU");
end

function AnvilToyBoxFilterDropDown_Initialize(self, level)
	local info = UIDropDownMenu_CreateInfo();
	info.keepShownOnClick = true;	

	if level == 1 then
		info.text = COLLECTED
		info.func = function(_, _, _, value)
						AnvilToyBox.firstCollectedToyID = 0;
						C_ToyBox.SetCollectedShown(value);
						AnvilToyBox_UpdatePages();
						AnvilToys:ResizeToyBox();
					end 
		info.checked = C_ToyBox.GetCollectedShown();
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.text = NOT_COLLECTED
		info.func = function(_, _, _, value)
						AnvilToyBox.firstCollectedToyID = 0;
						C_ToyBox.SetUncollectedShown(value);
						AnvilToyBox_UpdatePages();
						AnvilToys:ResizeToyBox();
					end 
		info.checked = C_ToyBox.GetUncollectedShown();
		info.isNotRadio = true;
		UIDropDownMenu_AddButton(info, level)

		info.checked = 	nil;
		info.isNotRadio = nil;
		info.func =  nil;
		info.hasArrow = true;
		info.notCheckable = true;

		info.text = SOURCES
		info.value = 1;
		UIDropDownMenu_AddButton(info, level)
	else
		if UIDROPDOWNMENU_MENU_VALUE == 1 then
			info.hasArrow = false;
			info.isNotRadio = true;
			info.notCheckable = true;				
		
			info.text = CHECK_ALL
			info.func = function()
							AnvilToyBox.firstCollectedToyID = 0;
							C_ToyBox.SetAllSourceTypeFilters(true);
							UIDropDownMenu_Refresh(AnvilToyBoxFilterDropDown, 1, 2);
							AnvilToyBox_UpdatePages();
							AnvilToys:ResizeToyBox();
						end
			UIDropDownMenu_AddButton(info, level)
			
			info.text = UNCHECK_ALL
			info.func = function()
							AnvilToyBox.firstCollectedToyID = 0;
							C_ToyBox.SetAllSourceTypeFilters(false);
							UIDropDownMenu_Refresh(AnvilToyBoxFilterDropDown, 1, 2);
							AnvilToyBox_UpdatePages();
							AnvilToys:ResizeToyBox();
						end
			UIDropDownMenu_AddButton(info, level)
		
			info.notCheckable = false;
			local numSources = C_PetJournal.GetNumPetSources();
			for i=1,numSources do
				if (i == 1 or i == 2 or i == 3 or i == 4 or i == 7 or i == 8) then
					info.text = _G["BATTLE_PET_SOURCE_"..i];
					info.func = function(_, _, _, value)
								AnvilToyBox.firstCollectedToyID = 0;
								C_ToyBox.SetSourceTypeFilter(i, value);
								AnvilToyBox_UpdatePages();
								AnvilToys:ResizeToyBox();
							end
					info.checked = function() return not C_ToyBox.IsSourceTypeFilterChecked(i) end;
					UIDropDownMenu_AddButton(info, level);
				end
			end
		end
	end
end
